<?php

namespace App\Http\Controllers;

use Log;
use Illuminate\Http\Request;
use App\Jobs\OrderCancelAndRefundJob;

class AutomatedOrderHandlingController extends Controller {

    private $shopifyWebhookSecret;

    public function __construct() {

        $this->shopifyWebhookSecret = env('SHOPIFY_WEBHOOK_SECRET', true);
    }

    /**
     * Shopify automated order cancel and refund
     */
    public function cancelAndRefundAutomatedOrder() {

        $hmac_header = $_SERVER['HTTP_X_SHOPIFY_HMAC_SHA256'];
        $requestData = file_get_contents('php://input');
        $verified = $this->verify_shopify_webhook($requestData, $hmac_header);
        $webhookStatus = var_export($verified, true);

        if ($webhookStatus) {

            $receivedData = json_decode($requestData);

            if(empty($receivedData->id)) {

                Log::channel('automatedOrderCancelAndRefundLog')->info('Shopify order create 
                webhook response is not correct, WebhookResponse:'.$requestData);
            } else {
                $queueResponse = dispatch(new OrderCancelAndRefundJob(
                $receivedData));
                ignore_user_abort(true);
                header("Content-Length: 0");
                header("Connection: Close");
                flush();
            }
        } else {

            Log::channel('automatedOrderCancelAndRefundLog')->info('Shopify order create webhook validation 
            failed, WebhookResponse:'.$requestData);
        }
    }

    /**
     * Shopify webhook verification
     */
    public function verify_shopify_webhook($data, $hmac_header) {

        $calculated_hmac = base64_encode(hash_hmac('sha256', $data, $this->shopifyWebhookSecret, true));
        return hash_equals($hmac_header, $calculated_hmac);
    }
}
