<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;

use DB;
use Log;
use Config;
use App\Http\ResponseHelper;
use App\Model\IterableApi;
use App\Model\Shopify\Product;

class IterableController extends Controller {

    public $iterableObj;

    public function __construct() {
       
        $this->productObj = new Product();
        $this->iterableUrl = env('ITERABLE_URL', true);
        $this->iterableApiKey = env('ITERABLE_API_KEY', true);
    }
    public function sendRequest() {
        Log::channel('abandonedCart')->info("Cron Started-----------------");

        $userDetails = DB::table('user_checkouts')
        ->select('id', 'user_email','checkout_recharge_token', 'checkout_updated_at', 'checkout_status', 'checkout_details', 'event_created_status', 'updated_at_server_time')
        ->whereIn('checkout_status', ['created', 'updated'])
        ->where('event_created_status', '=', '0' )
        ->get();
        if(empty($userDetails)) {
            Log::channel('abandonedCart')->info("Data is empty");    
            die;
        }
        $updateIDs = array();

        date_default_timezone_set('America/Los_Angeles');
        $currentTime =  date('Y-m-d H:i:s');        
        foreach($userDetails as $userDetail) {
            Log::channel('abandonedCart')->info("User email - " . $userDetail->user_email);
            $firstName = '';            
            $id = $userDetail->id;              
            $mail =  $userDetail->user_email;
            $eventUpdate = $userDetail->updated_at_server_time;
            $checkoutDetails = json_decode($userDetail->checkout_details);  
            $checkoutToken = $checkoutDetails->checkout->token;
            $totalPrice = isset($checkoutDetails->checkout->total_price)? $checkoutDetails->checkout->total_price: 0;
            $subTotalPrice = isset($checkoutDetails->checkout->subtotal_price)? $checkoutDetails->checkout->subtotal_price:'';
            $shipAddress = isset($checkoutDetails->checkout->shipping_address) ? $checkoutDetails->checkout->shipping_address: '';
            if($shipAddress) {
                $firstName = $shipAddress->first_name;
            }
            $eventStatus =  $userDetail->event_created_status;
            $checkoutCreatedAt = $checkoutDetails->checkout->created_at;
            $checkoutUpdatedAt = $checkoutDetails->checkout->updated_at;
            
            $discountCode = $checkoutDetails->checkout->discount_code;
            $lineItemDetails = $checkoutDetails->checkout->line_items;
            $lineItemProperties = array();
            foreach($lineItemDetails as $lineItemDetail) {            
                $productId = $lineItemDetail->product_id;
                $variantId = $lineItemDetail->variant_id;
                $chargeInterval = $lineItemDetail->charge_interval_frequency;
                $orderInterval = $lineItemDetail->order_interval_frequency;   
                $orderIntervalUnit = $lineItemDetail->order_interval_unit;
                $checkoutType = $lineItemDetail->type;
                $properties = $lineItemDetail->properties;
                $packCount = isset($properties->Packs)? (int)$properties->Packs: 0; 
                $oneTimePurchase = isset($lineItemDetail->one_time_purchase) ? $lineItemDetail->one_time_purchase: "";
                if(!$firstName) {
                    $firstName = isset($properties->first_name) ? $properties->first_name: ""; 
                } 
                $quizId = isset($properties->quiz_id) ? $properties->quiz_id: ""; 
                if(empty($properties)) {
                    Log::channel('abandonedCart')->info("empty properties" . $id);
                    continue;
                }
                foreach($properties as $key => $value) {   
                    if (strpos($key, 'Vitamin') !== false) {

                        $explodeValue = explode('|', $value);
                        $variantDetails = '';
                        $variantValue = 0;
                        
                        if (isset($explodeValue[3]) && !empty($explodeValue[3])) {
                        
                            $variantDetails = $this->productObj->getShopifyVariantByID($explodeValue[3]);                            
                            if(isset($variantDetails['errors']) && !empty($variantDetails['errors'])) {
                            
                                 Log::channel('abandonedCart')->info("Error in fetching variant price" . json_encode($variantDetails['errors']));
                            } 
                            $variantValue = isset($variantDetails['variant']['price'])? $variantDetails['variant']['price']: 0;   
                        }

                        $lineItemProperties[] = [
                            "name" => $key,  
                            "value" => isset($explodeValue[0])? trim($explodeValue[0]): '',
                            "variant_id" => isset($explodeValue[3])? trim($explodeValue[3]): '',
                            "quantity" => isset($explodeValue[2])? trim($explodeValue[2]): '',
                            "sku" => isset($explodeValue[4])? trim($explodeValue[4]): '',
                            "price" => trim($variantValue)
                        ];
                    }
                
                }
            }

            $timeDiff = $this->getTimeDiff($eventUpdate, $currentTime);
        
            //Checking the time difference is greater than 1 hour
            if($timeDiff >= 1 && $eventStatus !== 1) {
                $eventRequest = 
                        array(
                            "email" => $mail,
                            "eventName" => 'abandonedCart',
                            "dataFields" => array(
                                "checkout" => array(
                                    "checkoutToken" =>$checkoutToken,
                                    "email" => $mail,
                                    "firstName" => $firstName,
                                    "product_id" => $productId,
                                    "variant_id" => $variantId,
                                    "checkout_created_at" => $checkoutCreatedAt,
                                    "checkout_updated_at" => $checkoutUpdatedAt,
                                    "charge_interval_frequency" => $chargeInterval,
                                    "order_interval_frequency" => $orderInterval,
                                    "totalPrice" => $totalPrice,
                                    "subTotalPrice" => $subTotalPrice,
                                    "packCount" => $packCount,
                                    "order_interval_unit" => $orderIntervalUnit,
                                    "discount_code" => $discountCode,
                                    "properties" => $properties,
                                    "line_items" => $lineItemProperties,
                                    "quiz_id" => $quizId
                                )
                            )
                        );

                //checking type for OTP
                if($checkoutType == "ONETIME") {
                    unset($eventRequest["dataFields"]["checkout"]["charge_interval_frequency"]);
                    unset($eventRequest["dataFields"]["checkout"]["order_interval_frequency"]);
                    unset($eventRequest["dataFields"]["checkout"]["order_interval_unit"]);
                    if(!empty($oneTimePurchase)) {
                        $eventRequest["dataFields"]["checkout"]["one_time_purchase"] = $oneTimePurchase;
                    }
                }
                
                //Trigger Event
                $eventResponse = $this->createEvent(json_encode($eventRequest));
    
                if (isset($eventResponse->code) && strtolower(trim($eventResponse->code)) == 'success') {
                     Log::channel('abandonedCart')->info('Event triggered successfully Please check mail : '.$mail);
                    
                     $returnData = [
                         'stored' => true,
                         'message' => 'success'
                     ];
                     array_push($updateIDs, $id);
                 } else {
                    
                     Log::channel('abandonedCart')->info('Event not triggered');
                     $returnData = [
                         'error' => 'Event not triggered'
                     ];
                 }

            } else {
                Log::channel('abandonedCart')->info('No need to send mail notification to the user '.$mail);
            }
            
        }

         $updateUser = DB::table('user_checkouts')
             ->whereIn('id', $updateIDs)
             ->update(['event_created_status' => 1]);
        
        echo"done";die;
        
    }

    public function createEvent($data) {

        $path = $this->iterableUrl.'events/track?api_key='.$this->iterableApiKey;
        $method = 'POST';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    
    public function getTimeDiff($date1, $date2) {

        $diff = abs(strtotime($date2) - strtotime($date1));  

        $years = floor($diff / (365*60*60*24));

        $months = floor(($diff - $years * 365*60*60*24) 
                                    / (30*60*60*24));  

        $days = floor(($diff - $years * 365*60*60*24 -  
                    $months*30*60*60*24)/ (60*60*24)); 

        $hours = floor(($diff - $years * 365*60*60*24  
        - $months*30*60*60*24 - $days*60*60*24) / (60*60)); 

        return $hours;
    }

    public function call($method, $path, $data = [])
    {
        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        if ($method !== 'GET') {
            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }
        
        $response = curl_exec($curl);
        return $response;
    }
}
