<?php

namespace App\Http\Controllers;

use Log;
use Carbon\Carbon;
use App\Model\RechargeApi;
use App\Model\Shopify\Product;
use App\Model\IterableApi;
use Illuminate\Http\Request;

class UpcomingChargesValidityCheckingCron extends Controller {

    private $rechargeObj;
    private $productObj;
    private $iterableObj;
    private $logDirectory;
    private $logPath;
    private $logFiles;
    private $futureChargePeriod;
    private $productTitlePartToTrim = [];
    private $productTitlePartToRemove = [];
    private $productDetailsWithWrongPillName = [];
    private $totalProcessedChargeCount = 0;
    
    public function __construct() {

        $this->futureChargePeriod = 10;
        $this->productTitlePartToTrim = ["<sub>","</sub>"];
        $this->productTitlePartToRemove = ["<sup>"];
        $this->rechargeObj = new RechargeApi('v2General');
        $this->productObj = new Product();
        $this->iterableObj = new IterableApi();
    }

    /**
     * Upcoming charges validity checking cron
     */
    public function processChargeValidityChecking() {

        $shProductDetails = [];
        $updatedShProducts = [];
        $shSpaceAdjustedProducts = [];
        $minChargeDate = '';
        $maxChargeDate = '';
        Log::channel('upcomingChargeValidityCheckingLog')->info("Started the charge validity checking cron working");

        $shProductDetails = $this->getShopifyProductDetails();

        $updatedShProducts = $this->adjustShProductNames($shProductDetails);

        $minChargeDate = date('Y-m-d');
        $this->futureChargePeriod = (int)$this->futureChargePeriod;
        $maxChargeDate = date('Y-m-d',strtotime('+'.$this->futureChargePeriod.' days',strtotime($minChargeDate)));

        $this->getUpcomingChargesWithWrongPillName($minChargeDate, $maxChargeDate, $updatedShProducts);

        $this->sendSubscriptionValidityCheckedReport();
        Log::channel('upcomingChargeValidityCheckingLog')->info("Completed the charge validity checking cron working");
    }

    /**
     * Adjust Shopify product names
     */
    public function adjustShProductNames($shProductDetails) {

        $shUpdatedProducts = [];

        foreach ($shProductDetails as $shKey => $shValue) {

            $originalProductName = '';
            $productTitlePartUpdateStatus = 0;
            $modifiedProductName = '';
            $originalProductName = trim($shValue['product_title']);
            $modifiedProductName = $originalProductName;

            foreach ($this->productTitlePartToTrim as $tKey => $tValue) {

                if (strpos($modifiedProductName, $tValue)) {

                    $modifiedProductName = str_replace($tValue, "", $modifiedProductName);
                    $productTitlePartUpdateStatus = 1;
                }
            }

            foreach ($this->productTitlePartToRemove as $rKey => $rValue) {

                if (strpos($modifiedProductName, $rValue)) {

                    $productNameToRemove = '';
                    $productNameToRemove = $modifiedProductName;
                    $productNameToRemove = explode($rValue, $productNameToRemove);
                    if (!empty($productNameToRemove[1])) {

                        $explodedNamePart = explode(" ", $productNameToRemove[1]);

                        if (!empty($explodedNamePart[1])) {
                            $modifiedProductName = $productNameToRemove[0]." ".trim($explodedNamePart[1]);
                        } else {
                            $modifiedProductName = $productNameToRemove[0];
                        }
                    }
                    $productTitlePartUpdateStatus = 1;
                }
            }

            if ($productTitlePartUpdateStatus == 1 && !empty($modifiedProductName)) {

                $modifiedProductName = $this->purifyProductName($modifiedProductName);
                $shUpdatedProducts[] = $modifiedProductName;
            } else {
                $originalProductName = $this->purifyProductName($originalProductName);
                $shUpdatedProducts[] = $originalProductName;
            }
        }
        return $shUpdatedProducts;
    }

    /**
     * Shopify products purification
     */
    public function purifyProductName($productName) {

        $productName = preg_replace('/\s+/', '', $productName);
        $productName = trim($productName);
        $productName = strtolower($productName);
        return $productName;
    }

    /**
     * Get upcoming charge details from recharge
     */
    public function getUpcomingChargesWithWrongPillName($minDate, $maxDate, $shopifyProducts) {

        Log::channel('upcomingChargeValidityCheckingLog')->info("Started to read the queued charges");
        $queuedChargeWithWrongPillNameDetails = [];
        $chargeWithWrongLipDetails = [];
        $chargeDetails = [];
        $pageNo = 1;
        $nextPageLink = '';
        $limit  = 200;
        do {
            try {
                if (!empty($chargeDetails['link'])) {

                    $chargeDetails = $this->rechargeObj->getQueuedChargesByMaxMinDates($limit, $chargeDetails['link'], $minDate, $maxDate);

                 } else {

                    $chargeDetails = $this->rechargeObj->getQueuedChargesByMaxMinDates($limit, NULL, $minDate, $maxDate);
                }
            }
            catch (Exception $e) {
                $errorText = "An error occurred while fetching the queued charges from the recharge, Error: ".$e->getMessage();
                Log::channel('upcomingChargeValidityCheckingLog')->info($errorText);
            }

            if (isset($chargeDetails['charges']) && !empty($chargeDetails['charges']) && !empty($chargeDetails)) {
            
                foreach ($chargeDetails['charges'] as $charge) {

                    $this->totalProcessedChargeCount++;
                    $subscriptionId = '';
                    $emailAddress = '';
                    $rechargeCustomerId = '';
                    $chargePillsWithWrongName = [];
                    $subscriptionId = $charge['line_items'][0]['subscription_id'];
                    $emailAddress = $charge['email'];
                    $rechargeCustomerId = $charge['customer_id'];

                    if (isset($charge['line_items'][0]['properties']) && !empty($charge['line_items'][0]['properties'])) {

                        foreach ($charge['line_items'][0]['properties'] as $chargeKey => $chargeValue) {

                            $propertyName = !empty($chargeValue['name']) ? strtolower(trim($chargeValue['name'])) : '';
        
                            if (strpos($propertyName, 'vitamin') !== false) {
    
                                $propertyValue = !empty($chargeValue['value']) ? $chargeValue['value'] : '';
    
                                if (strpos($propertyValue, '|') !== false) {
    
                                    $explodePropertyValue = explode('|', $propertyValue);

                                    $explodeTempArray = [];
                                    foreach ($explodePropertyValue as $expKey => $expValue) {

                                        $expValue = trim($expValue);
                                        if (!empty($expValue)) {
                                            $explodeTempArray[] = trim($expValue);
                                        }
                                    }
                                    $explodePropertyValue = [];
                                    $explodePropertyValue = $explodeTempArray;
                                    
                                    if (count($explodePropertyValue) == 5) {

                                        $chargePill = preg_replace('/\s+/', '', $explodePropertyValue[0]);
                                        $chargePill = strtolower(trim($chargePill));

                                        if (!in_array($chargePill, $shopifyProducts)) {

                                            $chargeWithWrongLipDetails[$subscriptionId]['productName'][] = trim($chargeValue['name']);
                                            if (empty($chargeWithWrongLipDetails[$subscriptionId]['email'])) {
                                                $chargeWithWrongLipDetails[$subscriptionId]['email'] = $emailAddress;
                                                $chargeWithWrongLipDetails[$subscriptionId]['customerId'] = $rechargeCustomerId;
                                                $chargeWithWrongLipDetails[$subscriptionId]['subscriptionId'] = $subscriptionId;
                                            }
                                            $chargeWithWrongLipDetails[$subscriptionId]['error'][] = "Wrong product name";
                                        }
                                    } else {

                                        $chargeWithWrongLipDetails[$subscriptionId]['productName'][] = trim($chargeValue['name']);
                                        if (empty($chargeWithWrongLipDetails[$subscriptionId]['email'])) {
                                            $chargeWithWrongLipDetails[$subscriptionId]['email'] = $emailAddress;
                                            $chargeWithWrongLipDetails[$subscriptionId]['customerId'] = $rechargeCustomerId;
                                            $chargeWithWrongLipDetails[$subscriptionId]['subscriptionId'] = $subscriptionId;
                                        }
                                        $chargeWithWrongLipDetails[$subscriptionId]['error'][] = "Missing items in the LIP value";
                                    }
                                }
                            }
                        }
                    }
                }
            }
                            
            $nextPageLink = !empty($chargeDetails['next']) ? $chargeDetails['next'] : "";
            if (empty($chargeDetails['next'])) {
                break;
            }
            $chargeCount = !empty($chargeDetails['orders']) ? count($chargeDetails['orders']) : 0;
            $pageNo++;

        } while($chargeCount == $limit);

        $this->productDetailsWithWrongPillName = $chargeWithWrongLipDetails;
    }

    /**
     * Function to get all shopify products
    */
    public function getShopifyProductDetails() {

        $shopifyProducts  = [];
        $pageNo = 1;
        $limit  = 200;
        do {
            try {
                if(!empty($products['link'])) {

                    $products = $this->productObj->getShopifyAllProducts($limit,$products['link']);

                 } else {

                    $products = $this->productObj->getShopifyAllProducts($limit);
                    
                }
            }
            catch (Exception $e){

                echo PHP_EOL."Error form shopify when fetching the
                            product details and error is -  ".$e->getMessage();
                die;
            }

            $productsToSkip = array('4359176454226', '4359186350162', '4426715758674', '4716581486674', '4716582797394'); // Dev store
            //$productsToSkip = array('1809168695367', '4778895736903', '4778912219207', '4780554256455'); // Production
            
            if (!empty($products) && isset($products)) {
                
                foreach ($products['body'] as $product) {

                    if (in_array($product['id'], $productsToSkip)) {
                        continue;
                    }

                    if (!empty($product['id'])) {
                        $productDetails = [];
                        $productDetails = [
                            'product_title' => $product['title']
                            //'product_id' => $product['id']
                        ];
                        $shopifyProducts[] = $productDetails;
                    }
                }
                $productCount = count($products['body']);
            }

            $pageNo++;

            if (empty($products['link'])) {
                break;
            }
        } while($productCount == $limit);

        return $shopifyProducts;
    }

    /**
     * Sending validity checking report
     */
    public function sendSubscriptionValidityCheckedReport() {

        $iterableMailRequest = '';
        $reportSendingTime =  new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');        
        $reportSendingTime = $reportSendingTime->toDayDateTimeString();
        $totalSubscriptionProcessedForChecking = $this->totalProcessedChargeCount;
        $totalWrongSubscriptionCount = count($this->productDetailsWithWrongPillName);
	    $csvFilePath = env("CRON_LOG_CSV_PATH", true);

        if ($totalWrongSubscriptionCount > 30) {

            $this->setPaths();
            $this->getLogFilesList();
            $this->setWithLogFiles();

            $this->createReportIfAboveThirty();
        }

        $recipientEmails = 'testing@p80w.com';
        // $recipientEmails = 'mikegordon79@gmail.com';

        $subject = "Nurish-Dev Subscription Validity checker - ".$reportSendingTime;
        
        if ($totalWrongSubscriptionCount == 0) {
            Log::channel('upcomingChargeValidityCheckingLog')->info("This execution did not find any issues with the integrity of upcoming orders.");

            $commonContent = '';
            if ($totalSubscriptionProcessedForChecking == 1) {
                $commonContent .= $totalSubscriptionProcessedForChecking.' upcoming subscription record processed';
            } else {
                $commonContent .= $totalSubscriptionProcessedForChecking.' upcoming subscription records processed';
            }

            $commonContent .= '<br>This execution did not find any issues with the integrity of upcoming orders.';

            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_VALIDITY_CHECKING_CRON_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => '',
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
        } elseif ($totalWrongSubscriptionCount > 30) {

            $fileUrl = $csvFilePath.'/subscriptionValidityCheckingReport/'.$this->reportFileDirectory.'/'.$this->subscriptionValidityReport;

            $recordDetails = '';
            $commonContent = '';
            if ($totalSubscriptionProcessedForChecking == 1) {
                $commonContent .= $totalSubscriptionProcessedForChecking.' upcoming subscription record processed';
            } else {
                $commonContent .= $totalSubscriptionProcessedForChecking.' upcoming subscription records processed';
            }

            $recordDetails = 'This execution found '.$totalWrongSubscriptionCount.' issues with the integrity of upcoming orders.';
            $recordDetails = 'Please <a href="'.$fileUrl.'">click here</a> to download the file.';

            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_VALIDITY_CHECKING_CRON_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => $recordDetails,
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
            
        } elseif ($totalWrongSubscriptionCount > 0 && $totalWrongSubscriptionCount <= 30) {

            $recordDetails = '';
            $commonContent = '';
            if ($totalSubscriptionProcessedForChecking == 1) {
                $commonContent .= $totalSubscriptionProcessedForChecking.' upcoming subscription record processed';
            } else {
                $commonContent .= $totalSubscriptionProcessedForChecking.' upcoming subscription records processed';
            }

            $recordDetails = 'This execution found '.$totalWrongSubscriptionCount.' issues with the integrity of upcoming orders.'; 
            
            if (!empty($this->productDetailsWithWrongPillName)) {           
                $recordDetails .= "
                <table>
                    <tr>
                        <td>#</td>                
                        <td>Email</td>
                        <td>RechargeCustomerID</td> 
                        <td>SubscriptionID</td>
                        <td>InvalidLipVitamin</td>   
                        <td>Status</td>                       
                    </tr>            
                ";

                $i = 1;
                foreach ($this->productDetailsWithWrongPillName as $record) {
            
                    $productNameCount = 0;
                    $productNameCount = count($record['productName']);
                            
                    $recordDetails .= "
                    <tr>
                        <td>".$i."</td>                             
                        <td>".$record['email']."</td> 
                        <td>".$record['customerId']."</td>  
                        <td>".$record['subscriptionId']."</td>
                        <td>".$record['productName'][0]."</td>
                        <td>".$record['error'][0]."</td>
                    </tr>
                    ";
                    if ($productNameCount > 1) {

                        for ($j=1; $j < $productNameCount; $j++) { 

                            $recordDetails .= "
                            <tr>
                                <td> </td>                             
                                <td> </td> 
                                <td> </td>  
                                <td> </td>
                                <td>".$record['productName'][$j]."</td>
                                <td>".$record['error'][$j]."</td>
                            </tr>
                            ";
                        }
                    }

                    $i++;            
                }
                $recordDetails .= '</table>';
            }
            
            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_VALIDITY_CHECKING_CRON_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(                
                    "recordDetails" => $recordDetails,
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
        }
        
        if (!empty($iterableMailRequest)) {
            
            $mailTriggerResponse = $this->iterableObj->sendMailFromIterable(json_encode($iterableMailRequest));

            if (strtolower(trim($mailTriggerResponse->code)) == 'success') {           
                
                Log::channel('upcomingChargeValidityCheckingLog')->info("Successfully send the subscription validity 
                checking report mail, Email:".$recipientEmails);
    
            } else {
                
                Log::channel('upcomingChargeValidityCheckingLog')->info('Failed to send the subscription validity 
                checking report mail');
                Log::channel('upcomingChargeValidityCheckingLog')->info(json_encode($mailTriggerResponse));
            }
        }
    }

    /**
     * Create report if above 30
     */
    public function createReportIfAboveThirty() {

        if (isset($this->productDetailsWithWrongPillName) && !empty($this->productDetailsWithWrongPillName)) {

            $fileValidityChecker   = $this->logFiles['subscriptionsWithWrongPillName']['filename'];
            $filePointerChecker = fopen($fileValidityChecker, "a");

            foreach ($this->productDetailsWithWrongPillName as $pillKey => $pillValue) {

                $validityCheckReport = [];
                $productNameCount = 0;
                $productNameCount = count($pillValue['productName']);

                $validityCheckReport = array(
                    $pillValue['email'],
                    $pillValue['customerId'],
                    $pillValue['subscriptionId'],
                    $pillValue['productName'][0],  
                    $pillValue['error'][0]
                );
                fputcsv($filePointerChecker, $validityCheckReport);

                if ($productNameCount > 1) {

                    for ($j=1; $j < $productNameCount; $j++) { 

                        $validityCheckReport = array(
                            "",
                            "",
                            "",
                            $pillValue['productName'][$j],  
                            $pillValue['error'][$j]
                        );
                        fputcsv($filePointerChecker, $validityCheckReport);

                    }
                }
            }
        }
    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../public/subscriptionValidityCheckingReport/";
    }

    public function getLogFilesList() {
        
        $this->reportFileDirectory = date('dmY');

        $this->logPath = $this->logDirectory . $this->reportFileDirectory ;
        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);
        }
        
        $this->subscriptionValidityReport = "subscription-validity-checked-list-".date("Y_m_d_H_i").".csv";

        $this->logFiles = array(
            'subscriptionsWithWrongPillName' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->subscriptionValidityReport,
                "mode"     => "",
                "columns"  => array(
                    "Email",
                    "RechargeCustomerID",
                    "SubscriptionID",
                    "InvalidLipVitamin",                    
                    "Status"
                )
            )      
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {

            foreach ($this->logFiles as $logType=>$fileDetails){

                if ($fileDetails['type']=="csv"){
                    $mode = !empty($fileDetails['mode']) ? $fileDetails['mode'] : "w";

                    if ($mode != "r"){
                        $fileName    = $fileDetails['filename'];
                        $filePointer = fopen($fileName,$mode);
                        $columns     = $fileDetails['columns'];
                        fputcsv($filePointer,$columns);
                        chmod($fileName, 0777);
                        fclose($filePointer);
                    }
                }
            }
        }
    }

}
