<?php

namespace App\Http\Controllers;

use Log;
use Carbon\Carbon;
use App\Model\RechargeApi;
use App\Model\Shopify\Customer;
use App\Model\Shopify\Product;
use App\Model\Shopify\Order;
use App\Model\IterableApi;
use App\Helper\Helper;
use Illuminate\Http\Request;
use App\Http\Controllers\ShopifyMultiPass;

class giftPackController extends Controller {

    private $rechargeObj;
    private $helperObj;
    private $productObj;
    private $orderObj;
    private $customerObj;
    private $iterableObj;
    private $gifteeProcessedEvents = [];
    private $giftExpirationProcessedEvents = [];
    private $giftPurchasedDelayedShipment = [];
    private $giftPurchasedPendingClaim = [];
    private $newCustomersCreatedInGiftee = [];
    private $logDirectory;
    private $reportFileDirectory;
    private $logPath;
    private $gifteeEventsCreatedReport;
    private $giftExpirationEventsCreatedReport;
    private $giftPurchasedPendingClaimReport;
    private $giftPurchasedDelayedShipmentReport;
    private $logFiles;
    private $shopifyCustomerLink;
    
    public function __construct() {

        $this->rechargeObj = new RechargeApi('gift');
        $this->productObj = new Product();
        $this->orderObj = new Order();
        $this->customerObj = new Customer();
        $this->iterableObj = new IterableApi();
        $this->shopifyCustomerLink = env("SHOPIFY_CUSTOMER_LINK", true);
        //date_default_timezone_set('America/Los_Angeles');
    }

    /**
     * Update ship date tag to the order
     */
    public function setGiftRecipientOrderShipDate(Request $request) {

        $requestData = $request->input('data');

        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Invalid endpoint request");
            return $this->returnData(600, "error", $returnData);
        }

        $receivedData = json_decode($requestData);

        if (empty($receivedData->token) || empty($receivedData->shipDate)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: The token or shipDate is empty 
            in the endpoint request, endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        $token = !empty($receivedData->token) ? trim($receivedData->token) : '';
        $shipDate = !empty($receivedData->shipDate) ? trim($receivedData->shipDate) : '';
        $newPassword = !empty($receivedData->newPassword) ? trim($receivedData->newPassword) : '';
        $confirmPassword = !empty($receivedData->confirmPassword) ? trim($receivedData->confirmPassword) : '';

        if (empty($token)) {

            $returnData = [
                'error' => 'Required parameter token is missing in the endpoint request',
            ];
            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Required parameter token is missing 
            in the endpoint request, endpoint request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        if (empty($shipDate)) {

            $returnData = [
                'error' => 'Required parameter shipDate is missing in the endpoint request',
            ];
            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Required parameter shipDate is missing 
            in the endpoint request, endpoint request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        if (!empty($newPassword) || !empty($confirmPassword)) {

            if ($newPassword != $confirmPassword) {

                $returnData = [
                    'error' => 'Found password mismatch',
                ];
                Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Found password mismatch, 
                endpoint request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);
            }
        }

        $purchaserSubscriptionId = '';
        $recipientShopifyCustomerId = '';
        $decodedToken = $this->decodeEndpointRequest($token);

        if (strpos($decodedToken, ':') !== false) {

            $explodeToken = explode(":", $decodedToken);
            $purchaserSubscriptionId = $explodeToken[0];
            $recipientShopifyCustomerId = $explodeToken[1];
        }

        if (empty($purchaserSubscriptionId) || empty($recipientShopifyCustomerId)) {

            $returnData = [
                'error' => 'Failed to find the required details from the token',
            ];
            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to find the required details 
            from the token, endpoint request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        // Check the gift recipient is in recharge or not
        $rechargeCustomerDetails = $this->rechargeObj->getCustomerDetailsByShopifyCustomerId($recipientShopifyCustomerId);
        if (!empty($rechargeCustomerDetails->error) || !empty($rechargeCustomerDetails->errors)) {

            $rechargeError  = "Unknown error";
            if (!empty($rechargeCustomerDetails->error)) {

                $rechargeError = json_encode($rechargeCustomerDetails->error);
            } else if (!empty($rechargeCustomerDetails->errors)) {

                $rechargeError = json_encode($rechargeCustomerDetails->errors);
            } else {

                $rechargeError = 'Failed to get the customer details from recharge';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to get the customer details from 
            the recharge(ShopifyCustomerId:".$recipientShopifyCustomerId."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData);
        }

        // Get Shopify customer details by Id
        $shopifyCustomerDetails = $this->customerObj->getCustomerDetails($recipientShopifyCustomerId);
        if (!empty($shopifyCustomerDetails['error']) || !empty($shopifyCustomerDetails['errors']) ||
            empty($shopifyCustomerDetails) || empty($shopifyCustomerDetails['customer'])) {

            $shopifyError  = "Unknown error";
            if (!empty($shopifyCustomerDetails['error'])) {

                $shopifyError = json_encode($shopifyCustomerDetails['error']);
            } else if (!empty($shopifyCustomerDetails['errors'])) {

                $shopifyError = json_encode($shopifyCustomerDetails['errors']);
            } else if (empty($shopifyCustomerDetails) || empty($shopifyCustomerDetails['customer'])) {

                $shopifyError = 'Failed to get the customer details from Shopify';
            }
            $returnData = [
                'error' => $shopifyError
            ];

            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to get the customer details 
            from Shopify(CustomerId:".$recipientShopifyCustomerId."), Error: (".$shopifyError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $shopifyRecipientCustomerEmailFromInput = '';
        $shopifyCustomerExistingTags = '';
        $shopifyRecipientCustomerEmailFromInput = !empty($shopifyCustomerDetails['customer']['email']) ? 
        trim($shopifyCustomerDetails['customer']['email']) : '';
        $customerStatus = !empty($shopifyCustomerDetails['customer']['state']) ? 
        $shopifyCustomerDetails['customer']['state'] : '';
        $shopifyCustomerExistingTags = !empty($shopifyCustomerDetails['customer']['tags']) ? 
        $shopifyCustomerDetails['customer']['tags'] : '';

        if (trim($customerStatus) != 'enabled') {

            $returnData = [
                'error' => 'The gift recipient Shopify account is not enabled',
            ];
            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: The gift recipient Shopify account is 
            not enabled, endpoint request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        // Get charge details from recharge by subscriptionId
        $chargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionId($purchaserSubscriptionId);
        if (!empty($chargeDetails->error) || !empty($chargeDetails->errors) ||
            empty($chargeDetails) || empty($chargeDetails->charges)) {

            $rechargeError  = "Unknown error";
            if (!empty($chargeDetails->error)) {

                $rechargeError = json_encode($chargeDetails->error);
            } else if (!empty($chargeDetails->errors)) {

                $rechargeError = json_encode($chargeDetails->errors);
            } else if (empty($chargeDetails) || empty($chargeDetails->charges)) {

                $rechargeError = 'Failed to get the charge details from recharge';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to get the charge details from 
            the recharge(subscriptionId:".$purchaserSubscriptionId."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $shopifyRecipientCustomerEmailFromPurchaserCharge = '';
        $shopifyOrderId = !empty($chargeDetails->charges[0]->shopify_order_id) ? 
        $chargeDetails->charges[0]->shopify_order_id : '';
        $purchaserSubscriptionAddressId = !empty($chargeDetails->charges[0]->address_id) ? 
        $chargeDetails->charges[0]->address_id : '';
        $purchaserProperties = isset($chargeDetails->charges[0]->line_items[0]->properties) ? 
        $chargeDetails->charges[0]->line_items[0]->properties : '';
        $recipientShippingAddress = !empty($chargeDetails->charges[0]->shipping_address) ? 
        $chargeDetails->charges[0]->shipping_address : '';

        if (isset($purchaserProperties)) {

            foreach ($purchaserProperties as $propKey => $propValue) {

                if(trim($propValue->name) == 'giftee_email') {

                    $shopifyRecipientCustomerEmailFromPurchaserCharge = trim($propValue->value);
                }
            }
        }
        if (strtolower($shopifyRecipientCustomerEmailFromInput) != strtolower($shopifyRecipientCustomerEmailFromPurchaserCharge)) {

            $returnData = [
                'error' => 'Failed to match the gift recipient with the recipient in the purchaser order',
            ];
            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to match the gift recipient 
            with the recipient in the purchaser order, subscriptionId: ".$purchaserSubscriptionId.", orderId:".
            $shopifyOrderId);
            return $this->returnData(600, "error", $returnData);
        }
        if (empty($shopifyOrderId)) {

            $returnData = [
                'error' => 'Failed to get the order details from Shopify',
            ];
            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to get the Shopify orderId 
            from recharge charge, subscriptionId: ".$purchaserSubscriptionId);
            return $this->returnData(600, "error", $returnData);
        }

        // Get order details from Shopify
        $shopifyOrderDetails = $this->orderObj->getOrderDetailsById($shopifyOrderId);
        if (!empty($shopifyOrderDetails['error']) || !empty($shopifyOrderDetails['errors']) ||
            empty($shopifyOrderDetails) || empty($shopifyOrderDetails['order'])) {

            $shopifyError  = "Unknown error";
            if (!empty($shopifyOrderDetails['error'])) {

                $shopifyError = json_encode($shopifyOrderDetails['error']);
            } else if (!empty($shopifyOrderDetails['errors'])) {

                $shopifyError = json_encode($shopifyOrderDetails['errors']);
            } else if (empty($shopifyOrderDetails) || empty($shopifyOrderDetails['order'])) {

                $shopifyError = 'Failed to get the order details from Shopify';
            }
            $returnData = [
                'error' => $shopifyError
            ];

            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to get the order details 
            from Shopify(OrderId:".$shopifyOrderId."), Error: (".$shopifyError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $giftFirstOrderNumber = '';
        $giftFirstOrderNumber = !empty($shopifyOrderDetails['order']['name']) ? 
        trim($shopifyOrderDetails['order']['name']) : '';

        // Checking order tags
        $orderTags = !empty($shopifyOrderDetails['order']['tags']) ? 
        trim($shopifyOrderDetails['order']['tags']) : '';

        if (empty($orderTags)) {

            $returnData = [
                'error' => 'The Shopify order tags are empty',
            ];
            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: The Shopify order tags are empty, 
            subscriptionId: ".$purchaserSubscriptionId.", orderId:".$shopifyOrderId);
            return $this->returnData(600, "error", $returnData);
        }

        $shOrderTags = explode(",", $orderTags);
        $shOrderTags = array_map('trim', $shOrderTags);
        if (!in_array('unclaimed', $shOrderTags)) {

            $returnData = [
                'error' => 'Not found unclaimed tag in the order',
            ];
            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Not found unclaimed tag in the order, 
            subscriptionId: ".$purchaserSubscriptionId.", orderId:".$shopifyOrderId);
            return $this->returnData(600, "error", $returnData);
        }

        // Create a recharge account for the gift recipient if already not
        if (empty($rechargeCustomerDetails->customers) || !isset($rechargeCustomerDetails->customers)) {

            $rechargeCustomerCreateRequest = [];
            if (!empty($shopifyRecipientCustomerEmailFromPurchaserCharge) && !empty($recipientShippingAddress)) {

                $rechargeCustomerCreateRequest = array(
                    "billing_address1" => !empty($recipientShippingAddress->address1) ? 
                    trim($recipientShippingAddress->address1) : '',
                    "billing_address2" => !empty($recipientShippingAddress->address2) ? 
                    trim($recipientShippingAddress->address2) : '',
                    "billing_city" => !empty($recipientShippingAddress->city) ? 
                    trim($recipientShippingAddress->city) : '',
                    "billing_company" => !empty($recipientShippingAddress->company) ? 
                    trim($recipientShippingAddress->company) : '',
                    "billing_country" => !empty($recipientShippingAddress->country) ? 
                    trim($recipientShippingAddress->country) : '',
                    "billing_first_name" => !empty($recipientShippingAddress->first_name) ? 
                    trim($recipientShippingAddress->first_name) : '',
                    "billing_last_name" => !empty($recipientShippingAddress->last_name) ? 
                    trim($recipientShippingAddress->last_name) : '',
                    "billing_phone" => !empty($recipientShippingAddress->phone) ? 
                    trim($recipientShippingAddress->phone) : '',
                    "billing_province" => !empty($recipientShippingAddress->province) ? 
                    trim($recipientShippingAddress->province) : '',
                    "billing_zip" => !empty($recipientShippingAddress->zip) ? 
                    trim($recipientShippingAddress->zip) : '',
                    "email" => !empty($shopifyRecipientCustomerEmailFromPurchaserCharge) ? 
                    trim($shopifyRecipientCustomerEmailFromPurchaserCharge) : '',
                    "first_name" => !empty($recipientShippingAddress->first_name) ? 
                    trim($recipientShippingAddress->first_name) : '',
                    "last_name" => !empty($recipientShippingAddress->last_name) ? 
                    trim($recipientShippingAddress->last_name) : ''
                );

                $rechargeCustomerCreateResponse = $this->rechargeObj->createRechargeCustomer(
                json_encode($rechargeCustomerCreateRequest));
                if (!empty($rechargeCustomerCreateResponse->error) || !empty($rechargeCustomerCreateResponse->errors) ||
                    empty($rechargeCustomerCreateResponse) || empty($rechargeCustomerCreateResponse->customer)) {

                    $rechargeError  = "Unknown error";
                    if (!empty($rechargeCustomerCreateResponse->error)) {

                        $rechargeError = json_encode($rechargeCustomerCreateResponse->error);
                    } else if (!empty($rechargeCustomerCreateResponse->errors)) {

                        $rechargeError = json_encode($rechargeCustomerCreateResponse->errors);
                    } else if (empty($rechargeCustomerCreateResponse) || empty($rechargeCustomerCreateResponse->customer)) {

                        $rechargeError = 'Failed to create the customer in recharge';
                    }
                    $returnData = [
                        'error' => $rechargeError
                    ];

                    Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to create the customer in recharge
                    (subscriptionId:".$purchaserSubscriptionId.", CreateRequest:".json_encode($rechargeCustomerCreateRequest)."), 
                    Error: (".$rechargeError.")");
                    return $this->returnData(600, "error", $returnData);
                }

            } else {

                $returnData = [
                    'error' => 'Failed to find the gift recipient details to create the recharge account'
                ];
                Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to create the 
                customer in recharge due to the recipient email address or shipping address being empty 
                (PurchaserSubscriptionId:".$purchaserSubscriptionId.")");
                return $this->returnData(600, "error", $returnData);
            }
        }

        $shopifyCustomerTags = [];
        if (!empty($shopifyCustomerExistingTags)) {

            $shopifyCustomerTags = explode(",", $shopifyCustomerExistingTags);
            $shopifyCustomerTags[] = "giftSub:".$purchaserSubscriptionId;
        } else {

            $shopifyCustomerTags[] = "giftSub:".$purchaserSubscriptionId;
        }
        $shopifyCustomerTags[] = "passwordSet";
        $implodeShopifyTags = '';
        $implodeShopifyTags = implode(",", $shopifyCustomerTags);

        // Set customer tags and password
        if (!empty($newPassword) && !empty($confirmPassword)) {

            if (empty($recipientShopifyCustomerId)) {
                
                $returnData = [
                    'error' => 'Failed to find the Shopify customer Id of the gift recipient',
                ];
                Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to find the Shopify customer Id 
                of the gift recipient, subscriptionId: ".$purchaserSubscriptionId.", request:".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);
            }
            $customerUpdateRequest = [];
            $customerUpdateRequest = array(
                "customer" => array(
                    "id" => $recipientShopifyCustomerId,
                    "password" => $newPassword,
                    "password_confirmation" => $confirmPassword,
                    "tags" => $implodeShopifyTags,
                    "send_email_welcome" => false
                ),
            );

            // Update giftee Shopify account
            $shopifyCustomerUpdateResponse = $this->customerObj->updateShopifyCustomer(
                $recipientShopifyCustomerId, $customerUpdateRequest);

            if (!empty($shopifyCustomerUpdateResponse['error']) || !empty($shopifyCustomerUpdateResponse['errors']) ||
                empty($shopifyCustomerUpdateResponse)) {

                $shopifyError  = "Unknown error";
                if (!empty($shopifyCustomerUpdateResponse['error'])) {

                    $shopifyError = json_encode($shopifyCustomerUpdateResponse['error']);
                } else if (!empty($shopifyCustomerUpdateResponse['errors'])) {

                    $shopifyError = json_encode($shopifyCustomerUpdateResponse['errors']);
                } else if (empty($shopifyCustomerUpdateResponse)) {

                    $shopifyError = 'Failed to update the giftee account in Shopify';
                }
                $returnData = [
                    'error' => $shopifyError
                ];

                Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to update the password to 
                the giftee account in Shopify(customerUpdateRequest:".json_encode($customerUpdateRequest)."), Error: (".
                $shopifyError.")");
                return $this->returnData(600, "error", $returnData);
            }
        } else {

            $customerUpdateRequest = [];
            $customerUpdateRequest = array(
                "customer" => array(
                    "id" => $recipientShopifyCustomerId,
                    "tags" => $implodeShopifyTags
                ),
            );

            // Update giftee Shopify account
            $shopifyCustomerUpdateResponse = $this->customerObj->updateShopifyCustomer(
                $recipientShopifyCustomerId, $customerUpdateRequest);

            if (!empty($shopifyCustomerUpdateResponse['error']) || !empty($shopifyCustomerUpdateResponse['errors']) ||
                empty($shopifyCustomerUpdateResponse)) {

                $shopifyError  = "Unknown error";
                if (!empty($shopifyCustomerUpdateResponse['error'])) {

                    $shopifyError = json_encode($shopifyCustomerUpdateResponse['error']);
                } else if (!empty($shopifyCustomerUpdateResponse['errors'])) {

                    $shopifyError = json_encode($shopifyCustomerUpdateResponse['errors']);
                } else if (empty($shopifyCustomerUpdateResponse)) {

                    $shopifyError = 'Failed to update the giftee account in Shopify';
                }
                $returnData = [
                    'error' => $shopifyError
                ];

                Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to update the password to 
                the giftee account in Shopify(customerUpdateRequest:".json_encode($customerUpdateRequest)."), Error: (".
                $shopifyError.")");
                return $this->returnData(600, "error", $returnData);
            }
        }

        // Update Shopify order tag
        $updatedShopifyTags = '';
        if (!empty($orderTags)) {

            $updatedShopifyTags = $this->checkAndModifyShopifyOrderTags($orderTags, $shipDate);
        }

        $orderUpdateRequest = [];
        $orderUpdateRequest = array (
            "order" => array (
                "id" => $shopifyOrderId,
                "tags" => $updatedShopifyTags
            )
        );

        // Get order details from Shopify
        $shopifyOrderUpdateResponse = $this->orderObj->updateOrder($orderUpdateRequest, $shopifyOrderId);
        if (!empty($shopifyOrderUpdateResponse['error']) || !empty($shopifyOrderUpdateResponse['errors']) ||
            empty($shopifyOrderUpdateResponse)) {

            $shopifyError  = "Unknown error";
            if (!empty($shopifyOrderUpdateResponse['error'])) {

                $shopifyError = json_encode($shopifyOrderUpdateResponse['error']);
            } else if (!empty($shopifyOrderUpdateResponse['errors'])) {

                $shopifyError = json_encode($shopifyOrderUpdateResponse['errors']);
            } else if (empty($shopifyOrderUpdateResponse)) {

                $shopifyError = 'Failed to update the Shopify order';
            }
            $returnData = [
                'error' => $shopifyError
            ];

            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to update the Shopify order
            (orderUpdateRequest:".json_encode($orderUpdateRequest)."), Error: (".$shopifyError.")");
            return $this->returnData(600, "error", $returnData);
        }

        // Get all order details from recharge by address ID
        $allOrderDetails = $this->rechargeObj->getRechargeOrdersByAddressId($purchaserSubscriptionAddressId);
        if (!empty($allOrderDetails->error) || !empty($allOrderDetails->errors) ||
            empty($allOrderDetails) || empty($allOrderDetails->orders)) {

            $rechargeError  = "Unknown error";
            if (!empty($allOrderDetails->error)) {

                $rechargeError = json_encode($allOrderDetails->error);
            } else if (!empty($allOrderDetails->errors)) {

                $rechargeError = json_encode($allOrderDetails->errors);
            } else if (empty($allOrderDetails) || empty($allOrderDetails->orders)) {

                $rechargeError = 'Failed to get the order details from recharge';
            }

            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to get the order details from 
            the recharge(subscriptionId:".$purchaserSubscriptionId.", addressId:".$purchaserSubscriptionAddressId."),
            Error: (".$rechargeError.")");
        }

        // Adjust gift subscription queued orders scheduled_at date
        $queuedOrderShipmentDate = $shipDate;
        foreach ($allOrderDetails->orders as $orderKey => $orderValue) {

            if (trim(strtolower($orderValue->status)) == 'queued') {

                $queuedOrderId = '';
                $nextOrderScheduledDate = '';
                $newOrderScheduledDate = '';
                $payload = [];
                $nextOrderScheduledDate = date('Y-m-d', strtotime($queuedOrderShipmentDate));
                $orderShippingPeriodDifference = 28;
                $orderShippingPeriodDifference = (int)$orderShippingPeriodDifference;
                $newOrderScheduledDate = date('Y-m-d',strtotime('+'.$orderShippingPeriodDifference.' days',
                strtotime($nextOrderScheduledDate)));
                $queuedOrderId = trim($orderValue->id);

                $payload = array(
                    "scheduled_at" => $newOrderScheduledDate
                );
                sleep(1);
                $queuedOrderUpdateResponse = $this->rechargeObj->updateOrderScheduledDate($queuedOrderId, json_encode($payload));

                if (!empty($queuedOrderUpdateResponse->error) || !empty($queuedOrderUpdateResponse->errors) ||
                empty($queuedOrderUpdateResponse)) {

                    $rechargeError  = "Unknown error";
                    if (!empty($queuedOrderUpdateResponse->error)) {

                        $rechargeError = json_encode($queuedOrderUpdateResponse->error);
                    } else if (!empty($queuedOrderUpdateResponse->errors)) {

                        $rechargeError = json_encode($queuedOrderUpdateResponse->errors);
                    } else if (empty($queuedOrderUpdateResponse)) {

                        $rechargeError = 'Failed to update the order scheduled_at date';
                    }

                    $returnData = [
                        'error' => $rechargeError
                    ];

                    Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: Failed to update the order scheduled_at 
                    date(subscriptionId:".$purchaserSubscriptionId.", addressId: ".$purchaserSubscriptionAddressId.")");
                    return $this->returnData(600, "error", $returnData, $request->input('callback'));
                }
                $queuedOrderShipmentDate = $newOrderScheduledDate;
            }
        }

        // Creating giftee-order-notification event
        $giftSendersFirstName = '';
        $giftSendersLastName = '';
        $giftPackName = '';
        $giftRecipientEmail = '';
        $purchaserOrderProperties = [];
        foreach ($allOrderDetails->orders as $orderKey => $orderValue) {

            if (trim(strtolower($orderValue->status)) == 'success') {

                //$giftFirstOrderNumber = $orderValue->shopify_order_number;
                $purchaserOrderProperties = $orderValue->line_items[0]->properties;
            }
        }

        if (isset($purchaserOrderProperties)) {
            foreach ($purchaserOrderProperties as $orderPropKey => $orderPropValue) {

                switch ($orderPropValue->name) {

                    case (trim($orderPropValue->name) == 'gift_senders_first_name'):
                        $giftSendersFirstName = trim($orderPropValue->value);
                        break;

                    case (trim($orderPropValue->name) == 'gift_senders_last_name'):
                        $giftSendersLastName = trim($orderPropValue->value);
                        break;

                    case (trim($orderPropValue->name) == 'gift_pack_name'):
                        $giftPackName = trim($orderPropValue->value);
                        break;
                    
                    case (trim($orderPropValue->name) == 'giftee_email'):
                        $giftRecipientEmail = trim($orderPropValue->value);
                        break;
                }
            }
        }
        $eventCreateRequest = [];
        $eventCreateRequest = array(
            "events" => array(
            array("email" => strtolower($giftRecipientEmail),
            "eventName" => "giftee-order-notification",
            "dataFields" => array(
                "gift_senders_first_name" => $giftSendersFirstName,
                "gift_senders_last_name" => $giftSendersLastName,
                "gift_pack_name" => $giftPackName,
                "gift_pack_order_number" => $giftFirstOrderNumber
                )
              )
            )
        );

        $eventCreatedResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventCreateRequest));

        if(isset($eventCreatedResponse->successCount) || $eventCreatedResponse->successCount > 0) {

            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate(giftee-order-notification):
            Successfully created the event for Email (".strtolower($giftRecipientEmail).")");
        } else {
            Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate(giftee-order-notification):
            Failed to create the event for Email (".strtolower($giftRecipientEmail).")");
        }

        // Multi pass login
        $shipDateParameter = '';
        $shipDateParameter = $shipDate;
        $shipDateParameter = str_replace('-', '/', $shipDateParameter);
        $shipDateParameter = date('m/d/Y', strtotime($shipDateParameter));

         $customer_login_data = array(
            "email" => $shopifyRecipientCustomerEmailFromInput,
            "return_to" => env("MULTI_PASS_RETURN_TO", true)."&shipment_date=".$shipDateParameter
        );
        $multiPassToken = env('MULTI_PASS_TOKEN', true);
        $multiPassObj = new ShopifyMultiPass($multiPassToken);
        $token = $multiPassObj->generate_token($customer_login_data);

        $responseMessage = 'Successfully updated the giftee account and the order tags in Shopify';
        
        $returnData = [
            'message' => $responseMessage,
            'multiPassToken' => ($token != '') ? $token : ""
        ];

        Log::channel('giftPackLog')->info("setGiftRecipientOrderShipDate: ".$responseMessage);
        return $this->returnData(200, "success", $returnData);
    }

    /**
     * Check and modify Shopify order tags
     */
    public function checkAndModifyShopifyOrderTags($orderTags, $shipDate) {

        $explodeTags = explode(",", $orderTags);
        $explodeTags = array_map('trim', $explodeTags);
        //$explodeTags = array_map('strtolower', $explodeTags);
        foreach ($explodeTags as $tagKey => $tagValue) {

            if (strtolower(trim($tagValue)) == 'unclaimed') {
                unset($explodeTags[$tagKey]);
            }
        }
        $explodeTags[] = 'claimed';
        $explodeTags[] = 'shipDate:'.$shipDate;
        $implodedShopifyTags = implode(',', $explodeTags);
        return $implodedShopifyTags;
    }

    /**
     * Get all subscriptions(regular and gift subscriptions if any)
     */
    public function getAllSubscriptions(Request $request) {

        $requestData = $request->input('data');

        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            Log::channel('giftPackLog')->info("getAllSubscriptions: Invalid endpoint request");
            return $this->returnData(600, "error", $returnData);
        }

        $receivedData = json_decode($requestData);

        if ((empty($receivedData->customerId) && empty($receivedData->hash)) && 
        (empty($receivedData->subscriptionId) && empty($receivedData->email))) {

            $returnData = [
                'error' => 'Invalid endpoint request'
            ];            
            Log::channel('giftPackLog')->info("getAllSubscriptions: Invalid endpoint request, request:".$requestData);
            return $this->returnData(600, "error", $returnData);
        }

        $customerId = !empty($receivedData->customerId) ? trim($receivedData->customerId) : '';
        $customerHash = !empty($receivedData->hash) ? trim($receivedData->hash) : '';
        $subscriptionId = !empty($receivedData->subscriptionId) ? trim($receivedData->subscriptionId) : '';
        $recipientEmail = !empty($receivedData->email) ? trim($receivedData->email) : '';

        $endpointInputIdentifier = '';
        if (!empty($customerId) || !empty($customerHash)) {

            $endpointInputIdentifier = 'customerId';
            if (empty($customerId)) {

                $returnData = [
                    'error' => 'Required parameter customerId is missing in the endpoint request',
                ];
                Log::channel('giftPackLog')->info("getAllSubscriptions: Required parameter customerId is missing 
                in the endpoint request, endpoint request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);

            } else if (empty($customerHash)) {

                $returnData = [
                    'error' => 'Required parameter customerHash is missing in the endpoint request',
                ];
                Log::channel('giftPackLog')->info("getAllSubscriptions: Required parameter customerHash is missing 
                in the endpoint request, endpoint request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);
            }

        } else if (!empty($subscriptionId) || !empty($recipientEmail)) {

            $endpointInputIdentifier = 'subscriptionId';
            if (empty($subscriptionId)) {

                $returnData = [
                    'error' => 'Required parameter subscriptionId is missing in the endpoint request',
                ];
                Log::channel('giftPackLog')->info("getAllSubscriptions: Required parameter subscriptionId is missing 
                in the endpoint request, endpoint request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);

            } else if (empty($recipientEmail)) {

                $returnData = [
                    'error' => 'Required parameter recipientEmail is missing in the endpoint request',
                ];
                Log::channel('giftPackLog')->info("getAllSubscriptions: Required parameter recipientEmail is missing 
                in the endpoint request, endpoint request: ".json_encode($receivedData));
                return $this->returnData(600, "error", $returnData);
            }
        }
        
        $customerSubscriptionDetails = [];
        if ($endpointInputIdentifier == 'customerId') {

            $customerSubscriptionDetails = $this->getExistingCustomerSubscriptionDetails($receivedData, 
            $customerId, $customerHash);
            if ($customerSubscriptionDetails['status'] == 'failed') {

                Log::channel('giftPackLog')->info($customerSubscriptionDetails['logInfo']);
                return $this->returnData(600, "error", $customerSubscriptionDetails['response']);
            }

        } else if ($endpointInputIdentifier == 'subscriptionId') {

            $customerSubscriptionDetails = $this->getNewCustomerSubscriptionDetails($receivedData, $subscriptionId, 
            $recipientEmail);
            if ($customerSubscriptionDetails['status'] == 'failed') {

                Log::channel('giftPackLog')->info($customerSubscriptionDetails['logInfo']);
                return $this->returnData(600, "error", $customerSubscriptionDetails['response']);
            }
        }

        $subscriptionsToReturn = [];
        //Add additional details to the subscriptions
        $subscriptionsToReturn = $this->addMoreInfoToTheSubscriptions($customerSubscriptionDetails);

        $returnData = [
            'message' => 'Successfully get the subscription details',
            'subscriptions' => $subscriptionsToReturn['subscriptions']
        ];

        Log::channel('giftPackLog')->info("getAllSubscriptions: ".json_encode($returnData));
        return $this->returnData(200, "success", $returnData);
    }

    /**
     * Get existing customer subscriptions and gift subscriptions if any
     */
    public function getExistingCustomerSubscriptionDetails($receivedData, $customerId, $customerHash) {

        $returnDataArray = [];
        // Get recharge customer details by ID
        $rechargeCustomerDetails = $this->rechargeObj->getRechargeCustomerDetailsByID($customerId);
        if (!empty($rechargeCustomerDetails->error) || !empty($rechargeCustomerDetails->errors) ||
            empty($rechargeCustomerDetails)) {

            $rechargeError  = "Unknown error";
            if (!empty($rechargeCustomerDetails->error)) {

                $rechargeError = json_encode($rechargeCustomerDetails->error);
            } else if (!empty($rechargeCustomerDetails->errors)) {

                $rechargeError = json_encode($rechargeCustomerDetails->errors);
            } else if (empty($rechargeCustomerDetails)) {

                $rechargeError = 'Failed to get the customer details from recharge';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            $returnDataArray['status'] = "failed";
            $returnDataArray['logInfo'] = "getAllSubscriptions: Failed to get the customer details from 
            the recharge(customerId:".$customerId."), Error: (".$rechargeError.")";
            $returnDataArray['response'] = $returnData;
            return $returnDataArray;
        }

        $shopifyCustomerId = trim($rechargeCustomerDetails->customer->shopify_customer_id);
        $rechargeHashToken = trim($rechargeCustomerDetails->customer->hash);

        if ($rechargeHashToken != $customerHash) {

            $returnData = [
                'error' => 'Failed to authenticate the user'
            ];

            $returnDataArray['status'] = "failed";
            $returnDataArray['logInfo'] = "getAllSubscriptions: Failed to authenticate the user
            (The recharge hash tokens are mismatched), endpoint request: ".json_encode($receivedData);
            $returnDataArray['response'] = $returnData;
            return $returnDataArray;
        }
        
        // Get all the subscriptions by customerId
        $subscriptionDetails = $this->rechargeObj->getAllSubscriptionsByCustomerID($customerId);
        if (!empty($subscriptionDetails->error) || !empty($subscriptionDetails->errors) ||
            empty($subscriptionDetails)) {

            $rechargeError  = "Unknown error";
            if (!empty($subscriptionDetails->error)) {

                $rechargeError = json_encode($subscriptionDetails->error);
            } else if (!empty($subscriptionDetails->errors)) {

                $rechargeError = json_encode($subscriptionDetails->errors);
            } else if (empty($subscriptionDetails)) {

                $rechargeError = 'Failed to get the subscriptions from recharge by customerId';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            $returnDataArray['status'] = "failed";
            $returnDataArray['logInfo'] = "getAllSubscriptions: Failed to get the subscriptions from 
            recharge by customerId(customerId:".$customerId."), Error: (".$rechargeError.")";
            $returnDataArray['response'] = $returnData;
            return $returnDataArray;
        }
        //print_r($subscriptionDetails->subscriptions); die;
        // Get shopify details of a customer
        $shopifyCustomerDetails = $this->customerObj->getCustomerDetails($shopifyCustomerId);
        if (!empty($shopifyCustomerDetails['error']) || !empty($shopifyCustomerDetails['errors']) ||
            empty($shopifyCustomerDetails)) {

            $shError  = "Unknown error";
            if (!empty($shopifyCustomerDetails['error'])) {

                $shError = json_encode($shopifyCustomerDetails['error']);
            } else if (!empty($shopifyCustomerDetails['errors'])) {

                $shError = json_encode($shopifyCustomerDetails['errors']);
            } else if (empty($shopifyCustomerDetails)) {

                $shError = 'Failed to get the customer details from Shopify';
            }
            $returnData = [
                'error' => $shError
            ];

            $returnDataArray['status'] = "failed";
            $returnDataArray['logInfo'] = "getAllSubscriptions: Failed to get the customer details from Shopify
            (shCustomerId:".$shopifyCustomerId."), Error: (".$shError.")";
            $returnDataArray['response'] = $returnData;
            return $returnDataArray;
        }

        $customerTags = !empty($shopifyCustomerDetails['customer']['tags']) ? 
        trim($shopifyCustomerDetails['customer']['tags']) : '';
        $giftSubscriptionIdFromTags = [];
        $giftSubscriptionDetails = [];

        if (!empty($customerTags)) {

            $giftSubscriptionIdFromTags = $this->checkGiftSubscriptionsInTags($customerTags);
            if (!empty($giftSubscriptionIdFromTags)) {

                // Get subscription by ID
                foreach ($giftSubscriptionIdFromTags as $subKey => $subValue) {

                    $giftSubscriptions = $this->rechargeObj->getSubscription($subValue);
                    if (!empty($giftSubscriptions->error) || !empty($giftSubscriptions->errors) ||
                        empty($giftSubscriptions)) {

                        $rechargeError  = "Unknown error";
                        if (!empty($giftSubscriptions->error)) {

                            $rechargeError = json_encode($giftSubscriptions->error);
                        } else if (!empty($giftSubscriptions->errors)) {

                            $rechargeError = json_encode($giftSubscriptions->errors);
                        } else if (empty($giftSubscriptions)) {

                            $rechargeError = 'Failed to get the subscription details by subscriptionId';
                        }
                        $returnData = [
                            'error' => $rechargeError
                        ];

                        $returnDataArray['status'] = "failed";
                        $returnDataArray['logInfo'] = "getAllSubscriptions: Failed to get the subscription 
                        details by subscriptionId(subscriptionId:".$subValue."), Error: (".$rechargeError.")";
                        $returnDataArray['response'] = $returnData;
                        return $returnDataArray;
                    }
                    $giftSubscriptionDetails[] = $giftSubscriptions->subscription;
                    sleep(1);
                }
            }
        }

        if (empty($subscriptionDetails->subscriptions) && empty($giftSubscriptionDetails)) {

            $returnData = [
                'error' => 'No subscription found'
            ];

            $returnDataArray['status'] = "failed";
            $returnDataArray['logInfo'] = "getAllSubscriptions: No subscription found, 
            endpoint request: ".json_encode($receivedData);
            $returnDataArray['response'] = $returnData;
            return $returnDataArray;
        }

        $returnDataArray['status'] = "success";
        if (!empty($subscriptionDetails->subscriptions)) {

            $returnDataArray['subscriptions']['regular'] = $subscriptionDetails->subscriptions;
        }
        if (!empty($giftSubscriptionDetails)) {

            $returnDataArray['subscriptions']['gift'] = $giftSubscriptionDetails;
        }

        return $returnDataArray;
    }

    /**
     * Get subscription details of the new gift recipient
     */
    public function getNewCustomerSubscriptionDetails($receivedData, $subscriptionId, $recipientEmail) {
        
        $returnDataArray = [];
        $giftSubscriptionDetails = [];
        if (strpos($subscriptionId, ',') !== false) {

            $giftSubscriptionIds = [];

            $explodeSubIds = explode(",", $subscriptionId);
            $giftSubscriptionIds = array_map('trim', $explodeSubIds);

            foreach ($giftSubscriptionIds as $giftSubKey => $giftSubValue) {

                $giftSubscriptions = $this->rechargeObj->getSubscription($giftSubValue);
                if (!empty($giftSubscriptions->error) || !empty($giftSubscriptions->errors) ||
                    empty($giftSubscriptions)) {

                    $rechargeError  = "Unknown error";
                    if (!empty($giftSubscriptions->error)) {

                        $rechargeError = json_encode($giftSubscriptions->error);
                    } else if (!empty($giftSubscriptions->errors)) {

                        $rechargeError = json_encode($giftSubscriptions->errors);
                    } else if (empty($giftSubscriptions)) {

                        $rechargeError = 'Failed to get the subscription details by subscriptionId';
                    }
                    $returnData = [
                        'error' => $rechargeError
                    ];

                    $returnDataArray['status'] = "failed";
                    $returnDataArray['logInfo'] = "getAllSubscriptions: Failed to get the subscription 
                    details by subscriptionId(subscriptionId:".$giftSubValue."), Error: (".$rechargeError.")";
                    $returnDataArray['response'] = $returnData;
                    return $returnDataArray;
                }
                $giftSubscriptionDetails[] = $giftSubscriptions->subscription;
                sleep(1);
            }
        } else {

            $giftSubscriptions = $this->rechargeObj->getSubscription($subscriptionId);
            if (!empty($giftSubscriptions->error) || !empty($giftSubscriptions->errors) ||
                empty($giftSubscriptions)) {

                $rechargeError  = "Unknown error";
                if (!empty($giftSubscriptions->error)) {

                    $rechargeError = json_encode($giftSubscriptions->error);
                } else if (!empty($giftSubscriptions->errors)) {

                    $rechargeError = json_encode($giftSubscriptions->errors);
                } else if (empty($giftSubscriptions)) {

                    $rechargeError = 'Failed to get the subscription details by subscriptionId';
                }
                $returnData = [
                    'error' => $rechargeError
                ];

                $returnDataArray['status'] = "failed";
                $returnDataArray['logInfo'] = "getAllSubscriptions: Failed to get the subscription 
                details by subscriptionId(subscriptionId:".$giftSubValue."), Error: (".$rechargeError.")";
                $returnDataArray['response'] = $returnData;
                return $returnDataArray;
            }
            $giftSubscriptionDetails[] = $giftSubscriptions->subscription;
        }

        $returnDataArray['status'] = "success";
        $returnDataArray['subscriptions']['gift'] = $giftSubscriptionDetails;
        return $returnDataArray;
    }

    /**
     * Add more details to the subscriptions
     */
    public function addMoreInfoToTheSubscriptions($customerSubscriptionDetails) {

        $returnDataArray = [];
        //Add shipping address to the regular subscriptions
        if (!empty($customerSubscriptionDetails['subscriptions']['regular'])) {

            foreach ($customerSubscriptionDetails['subscriptions']['regular'] as $dataKey => $dataValue) {

                // Get all order details from recharge by address ID
                $allOrderDetails = $this->rechargeObj->getRechargeOrdersByAddressId($dataValue->address_id);
                if (!empty($allOrderDetails->error) || !empty($allOrderDetails->errors) ||
                    empty($allOrderDetails) || empty($allOrderDetails->orders)) {

                    $rechargeError  = "Unknown error";
                    if (!empty($allOrderDetails->error)) {

                        $rechargeError = json_encode($allOrderDetails->error);
                    } else if (!empty($allOrderDetails->errors)) {

                        $rechargeError = json_encode($allOrderDetails->errors);
                    } else if (empty($allOrderDetails) || empty($allOrderDetails->orders)) {

                        $rechargeError = 'Failed to get the order details from recharge';
                    }
                    $returnData = [
                        'error' => $rechargeError
                    ];

                    $returnDataArray['status'] = "failed";
                    $returnDataArray['logInfo'] = "getAllSubscriptions: Failed to get the order details by using 
                    the given addressId: ".$dataValue->address_id;
                    $returnDataArray['response'] = $returnData;
                    return $returnDataArray;
                }

                if (!empty($allOrderDetails->orders)) {

                    foreach ($allOrderDetails->orders as $orderKey => $orderValue) {

                        $orderStatus = !empty($orderValue->status) ? strtolower(trim($orderValue->status)) : '';
                        if ($orderStatus == 'queued') {
                            $customerSubscriptionDetails['subscriptions']['regular'][$dataKey]->shipping_address = 
                            (object) $orderValue->shipping_address;
                            $customerSubscriptionDetails['subscriptions']['regular'][$dataKey]->scheduled_at = 
                            $orderValue->scheduled_at;
                            break;
                        }
                        $customerSubscriptionDetails['subscriptions']['regular'][$dataKey]->shipping_address = 
                        (object) $orderValue->shipping_address;
                    }
                }
                sleep(1);
            }
        }

        //Add shipping address to the gift subscriptions
        if (!empty($customerSubscriptionDetails['subscriptions']['gift'])) {

            foreach ($customerSubscriptionDetails['subscriptions']['gift'] as $dataKey => $dataValue) {

                // Get all order details from recharge by address ID
                $allOrderDetails = $this->rechargeObj->getRechargeOrdersByAddressId($dataValue->address_id);
                if (!empty($allOrderDetails->error) || !empty($allOrderDetails->errors) ||
                    empty($allOrderDetails) || empty($allOrderDetails->orders)) {

                    $rechargeError  = "Unknown error";
                    if (!empty($allOrderDetails->error)) {

                        $rechargeError = json_encode($allOrderDetails->error);
                    } else if (!empty($allOrderDetails->errors)) {

                        $rechargeError = json_encode($allOrderDetails->errors);
                    } else if (empty($allOrderDetails) || empty($allOrderDetails->orders)) {

                        $rechargeError = 'Failed to get the charge details from recharge';
                    }
                    $returnData = [
                        'error' => $rechargeError
                    ];

                    $returnDataArray['status'] = "failed";
                    $returnDataArray['logInfo'] = "getAllSubscriptions: Failed to get the order details by using 
                    the given addressId: ".$dataValue->address_id;
                    $returnDataArray['response'] = $returnData;
                    return $returnDataArray;
                }

                if (!empty($allOrderDetails->orders)) {

                    foreach ($allOrderDetails->orders as $orderKey => $orderValue) {

                        $orderStatus = !empty($orderValue->status) ? strtolower(trim($orderValue->status)) : '';
                        if ($orderStatus == 'queued') {
                            $customerSubscriptionDetails['subscriptions']['gift'][$dataKey]->shipping_address = 
                            (object) $orderValue->shipping_address;
                            $customerSubscriptionDetails['subscriptions']['gift'][$dataKey]->scheduled_at = 
                            $orderValue->scheduled_at;
                            //break;
                        }
                        // $customerSubscriptionDetails['subscriptions']['gift'][$dataKey]->shipping_address = 
                        // (object) $orderValue->shipping_address;
                    }
                }
                sleep(1);
            }
        }

        return $customerSubscriptionDetails;
    }

    /**
     * Check gift subscription details in the customer tags
     */
    public function checkGiftSubscriptionsInTags($customerTags) {

        $giftSubIds = [];
        $explodeTags = explode(",", $customerTags);
        $explodeTags = array_map('trim', $explodeTags);
        foreach ($explodeTags as $tagKey => $tagValue) {

            if (strpos($tagValue, 'giftSub') !== false) {

                $explodeValue = explode(":", $tagValue);
                $giftSubIds[] = trim($explodeValue[1]);
            }
        }
        return $giftSubIds;
    }

    /**
     * Cron job to create giftee and 30 days remaining iterable events
     */
    public function processGiftSubscriptionEvents() {

        // Create giftee event
        $this->gifteeEventCreation();

        // Create gift_subscription_expiring
        $this->giftExpirationEventCreation();

        // Get gifts purchased pending claim details
        $this->giftsPurchasedPendingClaim();

        // Get gifts purchased delayed shipment details
        $this->giftsPurchasedDelayedShipment();

        // To send the report email 
        $this->sendReportForGiftPackEventCreation();
    }

    /**
     * Giftee event creation in the cron job
     */
    public function gifteeEventCreation() {

        $orders = [];
        $orderSearchTags = [];
        $today = date('Y-m-d');
        $previousDay = date('Y-m-d',strtotime("-1 days"));
        //$today = '2021-09-14';
        // $orderSearchTag = 'giftNotificationDate:'.$today;
        $giftOrdersPreviousDay = 'giftNotificationDate:'.$previousDay;
        $giftOrdersToday = 'giftNotificationDate:'.$today;
        $orderSearchTags = [$giftOrdersPreviousDay, $giftOrdersToday];
        $gifteeEventProcessedTemp = [];
        $shopifyGifteeCustomerId = '';
        $gifteeUrlIfGifteeWithActiveShopifyAccount = env('NURISH_GIFT_CLAIM_URL', true);
        $limit  = 200;
        Log::channel('giftCronLog')->info("Started Giftee event creation cron working");
        foreach ($orderSearchTags as $orderTagKey => $orderTagValue) {
            do {
                try {
                    if (!empty($orders['link'])) {

                        $orders = $this->orderObj->getShopifyOrdersByTag($limit, $orders['link'], $orderTagValue);

                    } else {

                        $orders = $this->orderObj->getShopifyOrdersByTag(
                            $limit, NULL, $orderTagValue);

                        if (empty($orders['body'])) {
                            Log::channel('giftCronLog')->info("No orders found with the ".$orderTagValue." Tag");
                        }
                    }
                }
                catch (Exception $e) {

                    Log::channel('giftCronLog')->info("An error occurred while fetching the
                    order details from Shopify, Error:".json_encode($e->getMessage()));
                    die();
                }
                
                //print_r($orders); die;
                if (!empty($orders) && isset($orders['body'])) {
                    
                    foreach ($orders['body'] as $order) {

                        if (!empty($order['id'])) {

                            $gifteeFirstName = '';
                            $gifteeLastName = '';
                            $giftNote = '';
                            $giftPurchaserFirstName = '';
                            $giftPurchaserLastName = '';
                            $gifteeActivationLink = '';
                            $giftPackName = '';
                            $giftNumberChargesUntilExpiration = '';
                            $gifteeEmail = '';
                            $chargeIntervalFrequency = '';
                            $token = '';

                            $orderTags = $order['tags'];
                            $existingOrderTags = explode(",", $orderTags);
                            $existingOrderTags = array_map('trim', $existingOrderTags);

                            if (in_array('giftee-notified', $existingOrderTags)) {
                                
                                Log::channel('giftCronLog')->info("Skipped: Already created the Giftee event for the orderId:".
                                $order['id']);
                                continue;
                            }
                            if (!in_array($orderTagValue, $existingOrderTags)) {

                                Log::channel('giftCronLog')->info("Skipped: Giftee order notification tag not found in the order, 
                                orderId:".$order['id']);
                                continue;
                            }
                            if (empty($order['line_items'][0]['properties'])) {

                                Log::channel('giftCronLog')->info("Skipped: The order line item properties are empty, 
                                orderId:".$order['id']);
                                continue;
                            }
                            foreach ($order['line_items'][0]['properties'] as $lipKey => $lipValue) {

                                switch ($lipValue['name']) {

                                    case (trim($lipValue['name']) == 'giftee_first_name'):
                                        $gifteeFirstName = !empty($lipValue['value']) ? 
                                        trim($lipValue['value']) : '';
                                        break;

                                    case (trim($lipValue['name']) == 'giftee_last_name'):
                                        $gifteeLastName = !empty($lipValue['value']) ? 
                                        trim($lipValue['value']) : '';
                                        break;
                
                                    case (trim($lipValue['name']) == 'gift_note'):
                                        $giftNote = !empty($lipValue['value']) ? 
                                        trim($lipValue['value']) : '';
                                        break;
                
                                    case (trim($lipValue['name']) == 'gift_senders_first_name'):
                                        $giftPurchaserFirstName = !empty($lipValue['value']) ? 
                                        trim($lipValue['value']) : '';
                                        break;
                
                                    case (trim($lipValue['name']) == 'gift_senders_last_name'):
                                        $giftPurchaserLastName = !empty($lipValue['value']) ? 
                                        trim($lipValue['value']) : '';
                                        break;
                
                                    case (trim($lipValue['name']) == 'gift_pack_name'):
                                        $giftPackName = !empty($lipValue['value']) ? 
                                        trim($lipValue['value']) : '';
                                        break;
                
                                    case (trim($lipValue['name']) == 'gift_number_charges_until_expiration'):
                                        $giftNumberChargesUntilExpiration = !empty($lipValue['value']) ? 
                                        trim($lipValue['value']) : '';
                                        break;

                                    case (trim($lipValue['name']) == 'giftee_email'):
                                        $gifteeEmail = !empty($lipValue['value']) ? 
                                        trim($lipValue['value']) : '';
                                        break;

                                    case (trim($lipValue['name']) == 'charge_interval_frequency'):
                                        $chargeIntervalFrequency = !empty($lipValue['value']) ? 
                                        trim($lipValue['value']) : '';
                                        break;
                                }
                            }

                            $shopifyOrderId = trim($order['id']);
                            $shopifyGiftPurchaserCustomerId = trim($order['customer']['id']);

                            // Get charge details from recharge by shopify order ID
                            $chargeDetails = $this->rechargeObj->getChargeDetailsByShopifyOrderId($shopifyOrderId);
                            if (!empty($chargeDetails->error) || !empty($chargeDetails->errors) ||
                                empty($chargeDetails) || empty($chargeDetails->charges)) {

                                $rechargeError  = "Unknown error";
                                if (!empty($chargeDetails->error)) {

                                    $rechargeError = json_encode($chargeDetails->error);
                                } else if (!empty($chargeDetails->errors)) {

                                    $rechargeError = json_encode($chargeDetails->errors);
                                } else if (empty($chargeDetails) || empty($chargeDetails->charges)) {

                                    $rechargeError = 'Failed to get the charge details by using the given orderId: '.$shopifyOrderId;
                                }
                                $returnData = [
                                    'error' => $rechargeError
                                ];

                                Log::channel('giftCronLog')->info("Failed to get the charge details from 
                                the recharge(OrderId:".$shopifyOrderId."), Error: (".$rechargeError.")");
                                continue;
                            }
                            $purchaserSubscriptionId = !empty($chargeDetails->charges[0]->line_items[0]->subscription_id) ? 
                            trim($chargeDetails->charges[0]->line_items[0]->subscription_id) : '';

                            $purchaserName = $giftPurchaserFirstName. ' ' .$giftPurchaserLastName;

                            $recipientShippingAddress = !empty($chargeDetails->charges[0]->shipping_address) ? 
                            $chargeDetails->charges[0]->shipping_address : '';

                            if (empty($purchaserSubscriptionId) || empty($gifteeEmail)) {

                                Log::channel('giftCronLog')->info("Skipped: The gift purchaser subscriptionId or giftee email is empty
                                (OrderId:".$shopifyOrderId.", SubscriptionId:".$purchaserSubscriptionId.", GifteEmail:".$gifteeEmail.")");
                                continue;
                            }

                            // Check giftee account email exists in Shopify
                            $customerDetails = $this->customerObj->getCustomerDetailBySearch($gifteeEmail);

                            if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

                                $error = "Unknown error";
                                if (!empty($customerDetails['error'])) {

                                    $error = json_encode($customerDetails['error']);
                                } elseif (!empty($customerDetails['errors'])) {
                                    
                                    $error = json_encode($customerDetails['errors']);
                                }
                                if (empty($error)) {

                                $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $gifteeEmail . ")";
                                }
                                $returnData = [
                                    "error" => $error
                                ];

                                Log::channel('giftCronLog')->info("Failed to get the customer details from Shopify(emailAddress:".
                                $gifteeEmail."), Error: (".$error.")");
                                continue;
                            }
                            
                            $gifteeAccountExistsStatus = 0;
                            $existingGiftRecipientDetails = '';
                            if (!empty($customerDetails['customers']) || !empty($customerDetails) || isset($customerDetails['customers'])) {

                                foreach ($customerDetails['customers'] as $customerKey => $customerValue) {

                                    if (strtolower(trim($gifteeEmail)) == strtolower(trim($customerValue['email']))) {

                                        $gifteeAccountExistsStatus = 1;
                                        $existingGiftRecipientDetails = $customerValue;
                                        break;
                                    }
                                }
                            }
                            
                            if ($gifteeAccountExistsStatus == 1) {

                                $existingCustomerAccountStatus = '';
                                $shopifyGiftRecipientCustomerId = $existingGiftRecipientDetails['id'];
                                $shopifyGifteeCustomerId = $existingGiftRecipientDetails['id'];
                                $existingCustomerAccountStatus = $existingGiftRecipientDetails['state'];
                                if (trim($existingCustomerAccountStatus) != 'enabled') {

                                    // Create giftee Shopify account activation_url
                                    $customerActivationUrl = $this->customerObj->createAccountActivationLink($shopifyGiftRecipientCustomerId);

                                    if (!empty($customerActivationUrl['error']) || !empty($customerActivationUrl['errors']) ||
                                        empty($customerActivationUrl)) {

                                        $shopifyError  = "Unknown error";
                                        if (!empty($customerActivationUrl['error'])) {

                                            $shopifyError = json_encode($customerActivationUrl['error']);
                                        } else if (!empty($customerActivationUrl['errors'])) {

                                            $shopifyError = json_encode($customerActivationUrl['errors']);
                                        } else if (empty($customerActivationUrl)) {

                                            $shopifyError = 'Failed to create the giftee account activation URL';
                                        }
                                        $returnData = [
                                            'error' => $shopifyError
                                        ];

                                        Log::channel('giftCronLog')->info("Failed to create the giftee account activation URL(customerId:".
                                        $shopifyGiftRecipientCustomerId."), Error: (".$shopifyError.")");
                                        continue;
                                    }
                                    $gifteeActivationLink = $gifteeUrlIfGifteeWithActiveShopifyAccount.'?redeemgift=true';
                                } else {

                                    // Get customer details from recharge by shopify customer ID
                                    $rechargeCustomerDetails = $this->rechargeObj->getCustomerDetailsByShopifyCustomerId($shopifyGiftRecipientCustomerId);
                                    if (!empty($rechargeCustomerDetails->error) || !empty($rechargeCustomerDetails->errors)) {

                                        $rechargeError  = "Unknown error";
                                        if (!empty($rechargeCustomerDetails->error)) {

                                            $rechargeError = json_encode($rechargeCustomerDetails->error);
                                        } else if (!empty($rechargeCustomerDetails->errors)) {

                                            $rechargeError = json_encode($rechargeCustomerDetails->errors);
                                        } else if (empty($rechargeCustomerDetails->customers)) {

                                            $rechargeError = 'Response from recharge is empty or failed to get the 
                                            customer details from recharge';
                                        }
                                        $returnData = [
                                            'error' => $rechargeError
                                        ];

                                        Log::channel('giftCronLog')->info("Failed to get the customer details from 
                                        the recharge(customerId:".$shopifyGiftRecipientCustomerId."), Error: (".$rechargeError.")");
                                        continue;
                                    }

                                    if (empty($rechargeCustomerDetails->customers) || !isset($rechargeCustomerDetails->customers)) {

                                        $gifteeActivationLink = $gifteeUrlIfGifteeWithActiveShopifyAccount.'?redeemgift=true';
                                    } else {
                                        $rechargeCustomerHashToken = trim($rechargeCustomerDetails->customers[0]->hash);
                                        $gifteeActivationLink = $gifteeUrlIfGifteeWithActiveShopifyAccount.
                                        '?hash='.$rechargeCustomerHashToken.'&redeemgift=true';
                                    }
                                }

                            } else {

                                $customerRandomPassword = '';
                                $customerRandomPassword = $this->createRandomPassword();

                                $customerCreateRequest = [];
                                $customerCreateRequest = array(
                                    "customer" => array(
                                        "first_name" => $gifteeFirstName,
                                        "last_name" => $gifteeLastName,
                                        "email" => $gifteeEmail,
                                        "password" => $customerRandomPassword,
                                        "password_confirmation" => $customerRandomPassword,
                                        "send_email_welcome" => false,
                                        "addresses" => array(
                                            $recipientShippingAddress
                                        )
                                    ),
                                );

                                // Create giftee Shopify account
                                $shopifyCustomerCreateResponse = $this->customerObj->createShopifyCustomer($customerCreateRequest);

                                if (!empty($shopifyCustomerCreateResponse['error']) || !empty($shopifyCustomerCreateResponse['errors']) ||
                                    empty($shopifyCustomerCreateResponse)) {

                                    $shopifyError  = "Unknown error";
                                    if (!empty($shopifyCustomerCreateResponse['error'])) {

                                        $shopifyError = json_encode($shopifyCustomerCreateResponse['error']);
                                    } else if (!empty($shopifyCustomerCreateResponse['errors'])) {

                                        $shopifyError = json_encode($shopifyCustomerCreateResponse['errors']);
                                    } else if (empty($shopifyCustomerCreateResponse)) {

                                        $shopifyError = 'Failed to create the giftee account in Shopify';
                                    }
                                    $returnData = [
                                        'error' => $shopifyError
                                    ];

                                    Log::channel('giftCronLog')->info("Failed to create the giftee account in Shopify(customerCreateRequest:".
                                    json_encode($customerCreateRequest)."), Error: (".$shopifyError.")");
                                    continue;
                                }

                                $shopifyGifteeCustomerId = $shopifyCustomerCreateResponse['customer']['id'];
                                $gifteeEmailInLower = '';
                                $gifteeEmailInLower = trim(strtolower($gifteeEmail));
                                $this->newCustomersCreatedInGiftee[$gifteeEmailInLower] = $shopifyGifteeCustomerId;
                                // Create giftee Shopify account activation_url
                                // $customerActivationUrl = $this->customerObj->createAccountActivationLink($shopifyCustomerCreateResponse['customer']['id']);

                                // if (!empty($customerActivationUrl['error']) || !empty($customerActivationUrl['errors']) ||
                                //     empty($customerActivationUrl)) {

                                //     $shopifyError  = "Unknown error";
                                //     if (!empty($customerActivationUrl['error'])) {

                                //         $shopifyError = json_encode($customerActivationUrl['error']);
                                //     } else if (!empty($customerActivationUrl['errors'])) {

                                //         $shopifyError = json_encode($customerActivationUrl['errors']);
                                //     } else if (empty($customerActivationUrl)) {

                                //         $shopifyError = 'Failed to create the giftee account activation URL';
                                //     }
                                //     $returnData = [
                                //         'error' => $shopifyError
                                //     ];

                                //     Log::channel('giftCronLog')->info("Failed to create the giftee account activation URL(customerId:".
                                //     $shopifyCustomerCreateResponse['customer']['id']."), Error: (".$shopifyError.")");
                                //     continue;
                                // }

                                $gifteeActivationLink = $gifteeUrlIfGifteeWithActiveShopifyAccount.'?redeemgift=true';
                            }

                            $token = $purchaserSubscriptionId.':'.$shopifyGifteeCustomerId;
                            for ($i = 0; $i < 5; $i++) { 

                                $token = base64_encode($token);
                            }

                            if (!empty($chargeIntervalFrequency)) {
                                $chargeIntervalFrequency = (int)$chargeIntervalFrequency / 28;
                            }

                            $gifteeIterableEventRequest = [];
                            $gifteeIterableEventRequest = array(
                                "events" => array(
                                array("email" => strtolower($gifteeEmail),
                                "eventName" => "giftee",
                                "dataFields" => array(
                                    "giftee_first_name" => $gifteeFirstName,
                                    "giftee_last_name" => $gifteeLastName,
                                    "gift_note" => $giftNote,
                                    "gift_purchaser_name" => $purchaserName,
                                    "gift_pack_name" => $giftPackName,
                                    "gift_charge_interval_frequency" => $chargeIntervalFrequency,
                                    "token" => $token,
                                    "giftee_activation_link" => $gifteeActivationLink
                                    )
                                )
                            )
                            );

                            $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($gifteeIterableEventRequest));

                            if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {

                                Log::channel('giftCronLog')->info("Giftee iterable event: - Successfully created the
                                event for Email(".strtolower($gifteeEmail).")");

                                $gifteeEventProcessedTemp = array(
                                    "gifteeName" => $gifteeFirstName.' '.$gifteeLastName,
                                    "gifteeEmail" => $gifteeEmail,
                                    "gifteeShopifyCustomerId" => $shopifyGifteeCustomerId,
                                    "purchaserShopifyCustomerId" => $shopifyGiftPurchaserCustomerId,
                                    "Status" => "Success"
                                );

                                $this->gifteeProcessedEvents[] = $gifteeEventProcessedTemp;
                                $gifteeEventProcessedTemp = [];
                                $updatedShopifyTags = '';
                                $shopifyTagsToUpdate = [];
                                $shopifyTagsToUpdate = $existingOrderTags;
                                $shopifyTagsToUpdate[] = 'giftee-notified';
                                $updatedShopifyTags = implode(",", $shopifyTagsToUpdate);

                                $orderUpdateRequest = [];
                                $orderUpdateRequest = array (
                                    "order" => array (
                                        "id" => $shopifyOrderId,
                                        "tags" => $updatedShopifyTags
                                    )
                                );

                                // Update order tags to Shopify
                                $shopifyOrderUpdateResponse = $this->orderObj->updateOrder($orderUpdateRequest, $shopifyOrderId);
                                if (!empty($shopifyOrderUpdateResponse['error']) || !empty($shopifyOrderUpdateResponse['errors']) ||
                                    empty($shopifyOrderUpdateResponse)) {

                                    $shopifyError  = "Unknown error";
                                    if (!empty($shopifyOrderUpdateResponse['error'])) {

                                        $shopifyError = json_encode($shopifyOrderUpdateResponse['error']);
                                    } else if (!empty($shopifyOrderUpdateResponse->errors)) {

                                        $shopifyError = json_encode($shopifyOrderUpdateResponse['errors']);
                                    } else if (empty($shopifyOrderUpdateResponse)) {

                                        $shopifyError = 'Failed to update the Shopify order';
                                    }
                                    $returnData = [
                                        'error' => $shopifyError
                                    ];

                                    Log::channel('giftCronLog')->info("Failed to update the Shopify order(orderUpdateRequest:".
                                    json_encode($orderUpdateRequest)."), Error: (".$shopifyError.")");
                                }

                            } else {
                                Log::channel('giftCronLog')->info("Giftee iterable event: - Failed to create the event 
                                for Email (".strtolower($gifteeEmail).")");

                                $gifteeEventProcessedTemp = array(
                                    "gifteeName" => $gifteeFirstName.' '.$gifteeLastName,
                                    "gifteeEmail" => $gifteeEmail,
                                    "gifteeShopifyCustomerId" => $shopifyGifteeCustomerId,
                                    "purchaserShopifyCustomerId" => $shopifyGiftPurchaserCustomerId,
                                    "Status" => "Failed"
                                );

                                $this->gifteeProcessedEvents[] = $gifteeEventProcessedTemp;
                                $gifteeEventProcessedTemp = [];
                            }
                        }
                    }
                    $orderCount = count($orders['body']);
                }
                if (empty($orders['link'])) {
                    break;
                }
            } while($orderCount == $limit);
        }

        Log::channel('giftCronLog')->info("Completed Giftee event creation cron working");
    }

    /**
     * Gift expiration event creation in the cron job
     */
    public function giftExpirationEventCreation() {

        $shOrders = [];
        $gifteeExpirationEventProcessedTemp = [];
        $currentDate = date('Y-m-d');
        //$currentDate = '2021-09-14';
        $limit  = 200;
        Log::channel('giftCronLog')->info("Started Gift subscription expiration event creation cron working");
        do {
            try {
                if (!empty($shOrders['link'])) {

                    $shOrders = $this->orderObj->getShopifyOrdersByDate($limit, $shOrders['link'], $currentDate);

                } else {

                    $shOrders = $this->orderObj->getShopifyOrdersByDate(
                        $limit, NULL, $currentDate);
                    
                    if (empty($shOrders['body'])) {
                        Log::channel('giftCronLog')->info("No orders were found in the ".$currentDate." date");
                    }
                }
            }
            catch (Exception $e){

                Log::channel('giftCronLog')->info("An error occurred while fetching the
                order details from Shopify by date, Error:".json_encode($e->getMessage()));
                die();
            }

            //print_r($shOrders); die;
            if (!empty($shOrders) && isset($shOrders['body'])) {

                foreach ($shOrders['body'] as $shOrder) {

                    if (!empty($shOrder['id'])) {

                        $shopifyOrderId = '';
                        $shopifyOrderId = trim($shOrder['id']);
                        $orderLineItemProperties = !empty($shOrder['line_items'][0]['properties']) ? 
                        $shOrder['line_items'][0]['properties'] : '';

                        if (empty($orderLineItemProperties)) {

                            Log::channel('giftCronLog')->info("Skipped: The order LIP is empty
                            (ShopifyOrderId:".$shOrder['id'].", Email:".$shOrder['email'].")");
                            continue;
                        }

                        $orderTags = $shOrder['tags'];
                        $existingOrderTags = explode(",", $orderTags);
                        $existingOrderTags = array_map('trim', $existingOrderTags);

                        if (in_array('gift-subscription-expiration-notified', $existingOrderTags)) {
                            
                            Log::channel('giftCronLog')->info("Skipped: Already created the gift_subscription_expiring 
                            event for the orderId:".$shOrder['id']);
                            continue;
                        }
                        
                        $subscriptionId = '';
                        $gifteeFirstName = '';
                        $gifteeLastName = '';
                        $gifteeEmail = '';
                        $giftPackName = '';
                        $numberOfCharges = '';
                        $chargeIntervalFrequency = '';
                        $shopifyOrderCustomerId = '';

                        foreach ($shOrder['line_items'][0]['properties'] as $lipKey => $lipValue) {

                            switch ($lipValue['name']) {

                                case (trim($lipValue['name']) == 'giftee_first_name'):
                                    $gifteeFirstName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'giftee_last_name'):
                                    $gifteeLastName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'giftee_email'):
                                    $gifteeEmail = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'gift_pack_name'):
                                    $giftPackName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'gift_number_charges_until_expiration'):
                                    $numberOfCharges = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'charge_interval_frequency'):
                                    $chargeIntervalFrequency = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;
                            }
                        }

                        $shopifyOrderCustomerId = $shOrder['customer']['id'];

                        if (empty($gifteeEmail) || empty($chargeIntervalFrequency)) {

                            Log::channel('giftCronLog')->info("Skipped: The order is not a gift order
                            (ShopifyOrderId:".$shOrder['id'].", Email:".$shOrder['email'].")");
                            continue;
                        }
                        
                        // Get charge Shopify order charge details from recharge by Shopify order ID
                        $orderChargeDetails = $this->rechargeObj->getChargeDetailsByShopifyOrderId($shopifyOrderId);
                        if (!empty($orderChargeDetails->error) || !empty($orderChargeDetails->errors) ||
                            empty($orderChargeDetails) || empty($orderChargeDetails->charges)) {

                            $shRechargeError  = "Unknown error";
                            if (!empty($orderChargeDetails->error)) {

                                $shRechargeError = json_encode($orderChargeDetails->error);
                            } else if (!empty($orderChargeDetails->errors)) {

                                $shRechargeError = json_encode($orderChargeDetails->errors);
                            } else if (empty($orderChargeDetails) || empty($orderChargeDetails->charges)) {

                                $shRechargeError = 'Failed to get the charge details by using the given orderId: '.$shopifyOrderId;
                            }
                            $returnData = [
                                'error' => $shRechargeError
                            ];

                            Log::channel('giftCronLog')->info("Failed to get the charge details from 
                            the recharge(OrderId:".$shopifyOrderId."), Error: (".$shRechargeError.")");
                            continue;
                        }

                        $rechargeSubscriptionId = !empty($orderChargeDetails->charges[0]->line_items[0]->subscription_id) ? 
                        trim($orderChargeDetails->charges[0]->line_items[0]->subscription_id) : '';
                        $subscriptionAddressId = !empty($orderChargeDetails->charges[0]->address_id) ? 
                        trim($orderChargeDetails->charges[0]->address_id) : '';
                        
                        // Get giftee Shopify account details
                        $customerDetails = $this->customerObj->getCustomerDetailBySearch($gifteeEmail);

                        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

                            $error = "Unknown error";
                            if (!empty($customerDetails['error'])) {

                                $error = json_encode($customerDetails['error']);
                            } elseif (!empty($customerDetails['errors'])) {
                                
                                $error = json_encode($customerDetails['errors']);
                            }
                            if (empty($error)) {

                            $error = "An error occurred while getting the customer details from Shopify (EmailID - " . $gifteeEmail . ")";
                            }
                            $returnData = [
                                "error" => $error
                            ];

                            Log::channel('giftCronLog')->info("Failed to get the customer details from Shopify(emailAddress:".
                            $gifteeEmail."), Error: (".$error.")");
                            continue;
                        }
                        
                        $gifteeShopifyCustomerId = '';
                        $existingGiftRecipientDetails = '';
                        if (!empty($customerDetails['customers']) || !empty($customerDetails) || isset($customerDetails['customers'])) {

                            foreach ($customerDetails['customers'] as $customerKey => $customerValue) {

                                if (strtolower(trim($gifteeEmail)) == strtolower(trim($customerValue['email']))) {

                                    $gifteeShopifyCustomerId = trim($customerValue['id']);
                                    break;
                                }
                            }
                        }

                        // Get all charge details from recharge by subscription ID
                        // $allChargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionID($rechargeSubscriptionId);
                        // if (!empty($allChargeDetails->error) || !empty($allChargeDetails->errors) ||
                        //     empty($allChargeDetails) || empty($allChargeDetails->charges)) {

                        //     $rechargeError  = "Unknown error";
                        //     if (!empty($allChargeDetails->error)) {

                        //         $rechargeError = json_encode($allChargeDetails->error);
                        //     } else if (!empty($allChargeDetails->errors)) {

                        //         $rechargeError = json_encode($allChargeDetails->errors);
                        //     } else if (empty($allChargeDetails) || empty($allChargeDetails->charges)) {

                        //         $rechargeError = 'Failed to get the charge details by using the given subscriptionId: '.$rechargeSubscriptionId;
                        //     }
                        //     $returnData = [
                        //         'error' => $rechargeError
                        //     ];

                        //     Log::channel('giftCronLog')->info("Failed to get the charge details from 
                        //     the recharge(subscriptionId:".$rechargeSubscriptionId."), Error: (".$rechargeError.")");
                        //     continue;
                        // }

                        // Get all order details from recharge by address ID
                        $allOrderDetails = $this->rechargeObj->getRechargeOrdersByAddressId($subscriptionAddressId);
                        if (!empty($allOrderDetails->error) || !empty($allOrderDetails->errors) ||
                            empty($allOrderDetails) || empty($allOrderDetails->orders)) {

                            $rechargeError  = "Unknown error";
                            if (!empty($allOrderDetails->error)) {

                                $rechargeError = json_encode($allOrderDetails->error);
                            } else if (!empty($allOrderDetails->errors)) {

                                $rechargeError = json_encode($allOrderDetails->errors);
                            } else if (empty($allOrderDetails) || empty($allOrderDetails->orders)) {

                                $rechargeError = 'Failed to get the order details from recharge';
                            }

                            $returnData = [
                                'error' => $rechargeError
                            ];

                            Log::channel('giftCronLog')->info("Failed to get the order details from 
                            the recharge(subscriptionId:".$rechargeSubscriptionId.", addressId:".$subscriptionAddressId."),
                            Error: (".$rechargeError.")");
                            continue;
                        }

                        $successOrderCount = 0;
                        foreach ($allOrderDetails->orders as $orderKey => $orderValue) {

                            if (strtolower(trim($orderValue->status)) == 'success') {

                                $successOrderCount++;
                            }
                        }

                        $chargeIntervalFrequency = (int)$chargeIntervalFrequency / 28;
                        $successOrderCount = (int)$successOrderCount;

                        if ($chargeIntervalFrequency == $successOrderCount) {

                            for ($i = 0; $i < 5; $i++) { 

                                $rechargeSubscriptionId = base64_encode($rechargeSubscriptionId);
                            }

                            $gifExpirationEventRequest = [];
                            $gifExpirationEventRequest = array(
                                "events" => array(
                                    array("email" => strtolower($gifteeEmail),
                                    "eventName" => "gift_subscription_expiring",
                                    "dataFields" => array(
                                        "subscription_id" => $rechargeSubscriptionId,
                                        "gift_pack_name"  => $giftPackName
                                        )
                                    )
                                )
                            );

                            $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($gifExpirationEventRequest));

                            if(isset($eventResponse->successCount) || $eventResponse->successCount > 0) {

                                Log::channel('giftCronLog')->info("Gift expiration iterable event: - Successfully created the
                                event for Email(".strtolower($gifteeEmail).")");

                                $gifteeExpirationEventProcessedTemp = array(
                                    "gifteeName" => $gifteeFirstName.' '.$gifteeLastName,
                                    "gifteeEmail" => $gifteeEmail,
                                    "gifteeShopifyCustomerId" => $gifteeShopifyCustomerId,
                                    "purchaserShopifyCustomerId" => $shopifyOrderCustomerId,
                                    "Status" => "Success"
                                );

                                $this->giftExpirationProcessedEvents[] = $gifteeExpirationEventProcessedTemp;
                                $gifteeExpirationEventProcessedTemp = [];

                                $updatedShopifyTags = '';
                                $shopifyTagsToUpdate = [];
                                $shopifyTagsToUpdate = $existingOrderTags;
                                $shopifyTagsToUpdate[] = 'gift-subscription-expiration-notified';
                                $updatedShopifyTags = implode(",", $shopifyTagsToUpdate);

                                $orderUpdateRequest = [];
                                $orderUpdateRequest = array (
                                    "order" => array (
                                        "id" => $shopifyOrderId,
                                        "tags" => $updatedShopifyTags
                                    )
                                );

                                // Update order tags to Shopify
                                $shopifyOrderUpdateResponse = $this->orderObj->updateOrder($orderUpdateRequest, $shopifyOrderId);
                                if (!empty($shopifyOrderUpdateResponse['error']) || !empty($shopifyOrderUpdateResponse['errors']) ||
                                    empty($shopifyOrderUpdateResponse)) {

                                    $shopifyError  = "Unknown error";
                                    if (!empty($shopifyOrderUpdateResponse['error'])) {

                                        $shopifyError = json_encode($shopifyOrderUpdateResponse['error']);
                                    } else if (!empty($shopifyOrderUpdateResponse->errors)) {

                                        $shopifyError = json_encode($shopifyOrderUpdateResponse['errors']);
                                    } else if (empty($shopifyOrderUpdateResponse)) {

                                        $shopifyError = 'Failed to update the Shopify order';
                                    }
                                    $returnData = [
                                        'error' => $shopifyError
                                    ];

                                    Log::channel('giftCronLog')->info("Failed to update the Shopify order(orderUpdateRequest:".
                                    json_encode($orderUpdateRequest)."), Error: (".$shopifyError.")");
                                }

                            } else {
                                Log::channel('giftCronLog')->info("Gift expiration iterable event: - Failed to create the event 
                                for Email (".strtolower($gifteeEmail).")");

                                $gifteeExpirationEventProcessedTemp = array(
                                    "gifteeName" => $gifteeFirstName.' '.$gifteeLastName,
                                    "gifteeEmail" => $gifteeEmail,
                                    "gifteeShopifyCustomerId" => $gifteeShopifyCustomerId,
                                    "purchaserShopifyCustomerId" => $shopifyOrderCustomerId,
                                    "Status" => "Failed"
                                );

                                $this->giftExpirationProcessedEvents[] = $gifteeExpirationEventProcessedTemp;
                                $gifteeExpirationEventProcessedTemp = [];
                            }
                        }
                    }
                }
                $shOrderCount = count($shOrders['body']);
            }
            if (empty($shOrders['link'])) {
                break;
            }
        } while($shOrderCount == $limit);

        Log::channel('giftCronLog')->info("Completed Gift subscription expiration event creation cron working");
    }

    /**
     * Get gift purchased pending claim details
     */
    public function giftsPurchasedPendingClaim() {
        
        $shOrders = [];
        $orderSearchTag = 'unclaimed';
        $limit  = 200;
        Log::channel('giftCronLog')->info("Started Gift purchased pending claim cron working");
        do {
            try {
                if (!empty($shOrders['link'])) {

                    $shOrders = $this->orderObj->getShopifyOrdersByTag($limit, $shOrders['link'], $orderSearchTag);

                } else {

                    $shOrders = $this->orderObj->getShopifyOrdersByTag($limit, NULL, $orderSearchTag);
                    
                    if (empty($shOrders['body'])) {
                        Log::channel('giftCronLog')->info("No orders were found with the ".$orderSearchTag." tag");
                    }
                }
            }
            catch (Exception $e){

                Log::channel('giftCronLog')->info("An error occurred while fetching the
                order details from Shopify by date, Error:".json_encode($e->getMessage()));
                die();
            }

            if (!empty($shOrders) && isset($shOrders['body'])) {

                foreach ($shOrders['body'] as $shOrder) {

                    if (!empty($shOrder['id'])) {

                        $shopifyOrderId = '';
                        $shopifyOrderId = trim($shOrder['id']);

                        $orderLineItemProperties = !empty($shOrder['line_items'][0]['properties']) ? 
                        $shOrder['line_items'][0]['properties'] : '';

                        if (empty($orderLineItemProperties)) {

                            Log::channel('giftCronLog')->info("Skipped: The order LIP is empty
                            (ShopifyOrderId:".$shOrder['id'].", Email:".$shOrder['email'].")");
                            continue;
                        }

                        if ($shOrder['fulfillment_status'] == 'fulfilled') {
                            Log::channel('giftCronLog')->info("Skipped: The order is fulfilled
                            (ShopifyOrderId:".$shOrder['id'].", Email:".$shOrder['email'].")");
                            continue;
                        }

                        $subscriptionId = '';
                        $gifteeFirstName = '';
                        $gifteeLastName = '';
                        $gifteeEmail = '';
                        $inviteSendDate = '';
                        $shopifyOrderCustomerId = '';

                        foreach ($shOrder['line_items'][0]['properties'] as $lipKey => $lipValue) {

                            switch ($lipValue['name']) {

                                case (trim($lipValue['name']) == 'giftee_first_name'):
                                    $gifteeFirstName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'giftee_last_name'):
                                    $gifteeLastName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'giftee_email'):
                                    $gifteeEmail = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;
                                
                                case (trim($lipValue['name']) == 'gift_notification_date'):
                                    $inviteSendDate = !empty($lipValue['value']) ? 
                                    date('Y-m-d', strtotime(trim($lipValue['value']))) : '';
                                    break;
                            }
                        }

                        $shopifyOrderCustomerId = $shOrder['customer']['id'];

                        if (empty($gifteeEmail)) {

                            Log::channel('giftCronLog')->info("Skipped: The order is not a gift order
                            (ShopifyOrderId:".$shOrder['id'].", Email:".$shOrder['email'].")");
                            continue;
                        }

                        // Get giftee Shopify account details
                        $customerDetails = $this->customerObj->getCustomerDetailBySearch($gifteeEmail);

                        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

                            $error = "Unknown error";
                            if (!empty($customerDetails['error'])) {

                                $error = json_encode($customerDetails['error']);
                            } else if (!empty($customerDetails['errors'])) {
                                
                                $error = json_encode($customerDetails['errors']);
                            }
                            if (empty($error)) {

                                $error = "An error occurred while getting the customer details from Shopify";
                            }
                            $returnData = [
                                "error" => $error
                            ];

                            Log::channel('giftCronLog')->info("Failed to get the customer details from Shopify(
                            EmailAddress:".$gifteeEmail.", Error:".$error.")");
                            continue;
                        }
                        
                        $gifteeShopifyCustomerId = '';
                        if (!empty($customerDetails['customers']) || !empty($customerDetails) || 
                        isset($customerDetails['customers'])) {

                            foreach ($customerDetails['customers'] as $customerKey => $customerValue) {

                                if (strtolower(trim($gifteeEmail)) == strtolower(trim($customerValue['email']))) {

                                    $gifteeShopifyCustomerId = trim($customerValue['id']);
                                    break;
                                }
                            }
                        }

                        $gifteeEmailInLower = '';
                        if (empty($gifteeShopifyCustomerId) && isset($this->newCustomersCreatedInGiftee)) {

                            $gifteeEmailInLower = trim(strtolower($gifteeEmail));
                            $gifteeShopifyCustomerId = !empty($this->newCustomersCreatedInGiftee[$gifteeEmailInLower]) ?
                            $this->newCustomersCreatedInGiftee[$gifteeEmailInLower] : '';
                        }

                        $giftPurchasedPendingClaimArray = [];
                        $giftPurchasedPendingClaimArray = array(
                            "gifteeName" => $gifteeFirstName.' '.$gifteeLastName,
                            "gifteeEmail" => $gifteeEmail,
                            "gifteeShopifyCustomerId" => $gifteeShopifyCustomerId,
                            "purchaserShopifyCustomerId" => $shopifyOrderCustomerId,
                            "inviteSendDate" => $inviteSendDate,
                        );
                        $this->giftPurchasedPendingClaim[] = $giftPurchasedPendingClaimArray;
                    }
                }
                $shOrderCount = count($shOrders['body']);
            }
            if (empty($shOrders['link'])) {
                break;
            }
        } while($shOrderCount == $limit);

        Log::channel('giftCronLog')->info("Completed Gift purchased pending claim cron working");   
    }

    /**
     * Get gift purchased delayed Shipment details
     */
    public function giftsPurchasedDelayedShipment() {

        $shOrders = [];
        $orderSearchTag = 'claimed';
        $limit  = 200;
        Log::channel('giftCronLog')->info("Started Gift purchased delayed Shipment cron working");
        do {
            try {
                if (!empty($shOrders['link'])) {

                    $shOrders = $this->orderObj->getShopifyOrdersByTag($limit, $shOrders['link'], $orderSearchTag);

                } else {

                    $shOrders = $this->orderObj->getShopifyOrdersByTag($limit, NULL, $orderSearchTag);
                    
                    if (empty($shOrders['body'])) {
                        Log::channel('giftCronLog')->info("No orders were found with the ".$orderSearchTag." tag");
                    }
                }
            }
            catch (Exception $e){

                Log::channel('giftCronLog')->info("An error occurred while fetching the
                order details from Shopify by date, Error:".json_encode($e->getMessage()));
                die();
            }

            if (!empty($shOrders) && isset($shOrders['body'])) {

                foreach ($shOrders['body'] as $shOrder) {

                    if (!empty($shOrder['id'])) {

                        $shopifyOrderId = '';
                        $shopifyOrderId = trim($shOrder['id']);

                        $orderLineItemProperties = !empty($shOrder['line_items'][0]['properties']) ? 
                        $shOrder['line_items'][0]['properties'] : '';

                        if (empty($orderLineItemProperties)) {

                            Log::channel('giftCronLog')->info("Skipped: The order LIP is empty
                            (ShopifyOrderId:".$shOrder['id'].", Email:".$shOrder['email'].")");
                            continue;
                        }

                        $orderTags = !empty($shOrder['tags']) ? $shOrder['tags'] : '';
                        if (strpos($orderTags, 'shipDate:') === false) {
                            Log::channel('giftCronLog')->info("Skipped: The order does not have the shipDate tag 
                            (ShopifyOrderId:".$shOrder['id'].", Email:".$shOrder['email'].")");
                            continue;
                        }

                        if ($shOrder['fulfillment_status'] == 'fulfilled') {
                            Log::channel('giftCronLog')->info("Skipped: The order is fulfilled
                            (ShopifyOrderId:".$shOrder['id'].", Email:".$shOrder['email'].")");
                            continue;
                        }

                        $shipDate = '';
                        if (!empty($orderTags)) {

                            $existingOrderTags = explode(",", $orderTags);
                            $existingOrderTags = array_map('trim', $existingOrderTags);
                            foreach ($existingOrderTags as $tag) {
                                if (strpos($tag, 'shipDate:') !== false) {
                                    $shipDateArray = explode(':', $tag);
                                    $shipDate = date('Y-m-d', strtotime(trim($shipDateArray[1])));
                                }
                            }
                        }

                        $subscriptionId = '';
                        $gifteeFirstName = '';
                        $gifteeLastName = '';
                        $gifteeEmail = '';
                        $numberOfCharges = '';

                        foreach ($shOrder['line_items'][0]['properties'] as $lipKey => $lipValue) {

                            switch ($lipValue['name']) {

                                case (trim($lipValue['name']) == 'giftee_first_name'):
                                    $gifteeFirstName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'giftee_last_name'):
                                    $gifteeLastName = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;

                                case (trim($lipValue['name']) == 'giftee_email'):
                                    $gifteeEmail = !empty($lipValue['value']) ? 
                                    trim($lipValue['value']) : '';
                                    break;
                            }
                        }

                        $shopifyOrderCustomerId = $shOrder['customer']['id'];

                        if (empty($gifteeEmail)) {

                            Log::channel('giftCronLog')->info("Skipped: The order is not a gift order
                            (ShopifyOrderId:".$shOrder['id'].", Email:".$shOrder['email'].")");
                            continue;
                        }

                        // Get giftee Shopify account details
                        $customerDetails = $this->customerObj->getCustomerDetailBySearch($gifteeEmail);

                        if (!empty($customerDetails['errors']) || !empty($customerDetails['error'])) {

                            $error = "Unknown error";
                            if (!empty($customerDetails['error'])) {

                                $error = json_encode($customerDetails['error']);
                            } elseif (!empty($customerDetails['errors'])) {
                                
                                $error = json_encode($customerDetails['errors']);
                            }
                            if (empty($error)) {

                                $error = "An error occurred while getting the customer details from Shopify";
                            }
                            $returnData = [
                                "error" => $error
                            ];

                            Log::channel('giftCronLog')->info("Failed to get the customer details from Shopify
                            (EmailAddress:".$gifteeEmail.", Error: ".$error.")");
                            continue;
                        }
                        
                        $gifteeShopifyCustomerId = '';
                        if (!empty($customerDetails['customers']) || !empty($customerDetails) || 
                        isset($customerDetails['customers'])) {

                            foreach ($customerDetails['customers'] as $customerKey => $customerValue) {

                                if (strtolower(trim($gifteeEmail)) == strtolower(trim($customerValue['email']))) {

                                    $gifteeShopifyCustomerId = trim($customerValue['id']);
                                    break;
                                }
                            }
                        }

                        $giftPurchasedDelayedShipmentArray = [];
                        $giftPurchasedDelayedShipmentArray = array(
                            "gifteeName" => $gifteeFirstName.' '.$gifteeLastName,
                            "gifteeEmail" => $gifteeEmail,
                            "gifteeShopifyCustomerId" => $gifteeShopifyCustomerId,
                            "purchaserShopifyCustomerId" => $shopifyOrderCustomerId,
                            "gifteeShipmentDate" => $shipDate,
                        );
                        $this->giftPurchasedDelayedShipment[] = $giftPurchasedDelayedShipmentArray;
                    }
                }
                $shOrderCount = count($shOrders['body']);
            }
            if (empty($shOrders['link'])) {
                break;
            }
        } while($shOrderCount == $limit);
        
        Log::channel('giftCronLog')->info("Completed Gift delayed shipment cron working");   
    }

    /**
     * Gift pack cron report email
     */
    public function sendReportForGiftPackEventCreation() {
        
        $iterableMailRequest = '';
        $reportSendingTime =  new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');        
        $reportSendingTime = $reportSendingTime->toDayDateTimeString();
        $gifteeEventCreatedCount = 0;
        $giftExpirationEventCreatedCount = 0;
        $giftPurchasedPendingClaimCount = 0;
        $giftPurchasedDelayedShipmentCount = 0;
        $gifteeEventCreatedCount = count($this->gifteeProcessedEvents);
        $giftExpirationEventCreatedCount = count($this->giftExpirationProcessedEvents);
        $giftPurchasedPendingClaimCount = count($this->giftPurchasedPendingClaim);
        $giftPurchasedDelayedShipmentCount = count($this->giftPurchasedDelayedShipment);
        $csvFilePath = env("GIFT_EVENT_LOG_CSV_PATH", true);

        $csvFileName = '';

        if ($gifteeEventCreatedCount > 30 || $giftExpirationEventCreatedCount > 30 || 
        $giftPurchasedPendingClaimCount > 30 || $giftPurchasedDelayedShipmentCount > 30) {

            $this->setPaths();
            $this->getLogFilesList();
            $this->setWithLogFiles();

            // CSV creation with the giftee and gift expiration data if greater than 30
            if ($gifteeEventCreatedCount > 30) {

                $csvFileName = 'gifteeEventProcessed';
                $this->createReportIfAboveThirty($this->gifteeProcessedEvents, $csvFileName);
            }
            if ($giftExpirationEventCreatedCount > 30) {

                $csvFileName = 'giftExpireEventProcessed';
                $this->createReportIfAboveThirty($this->giftExpirationProcessedEvents, $csvFileName);
            }

            // CSV creation with the giftPurchasedPendingClaim and giftPurchasedDelayedShipment data if greater than 30
            if ($giftPurchasedPendingClaimCount > 30) {

                $csvFileName = 'giftPendingClaimProcessed';
                $this->createReportIfAboveThirty($this->giftPurchasedPendingClaim, $csvFileName, 'inviteSendDate');
            }
            if ($giftPurchasedDelayedShipmentCount > 30) {

                $csvFileName = 'giftDelayedShipmentProcessed';
                $this->createReportIfAboveThirty($this->giftPurchasedDelayedShipment, $csvFileName, 'gifteeShipmentDate');
            }
        }
        
        $commonContent = '';
        $recordDetails = '';
        $recordDetails2 = '';
        $recordDetailsGifteeTemp = '';
        $recordDetailsExpirationTemp = '';
        $giftPurchasedPendingClaimTemp = '';
        $giftPurchasedDelayedShipmentTemp = '';


        // Create a table with the giftee details
        if ($gifteeEventCreatedCount > 0 && $gifteeEventCreatedCount <= 30) {

            $recordDetailsGifteeTemp = $this->createReportIfBelowThirty($this->gifteeProcessedEvents, 'gifteeEvents');
        }

        // Create a table with the giftee expiration details
        if ($giftExpirationEventCreatedCount > 0 && $giftExpirationEventCreatedCount <= 30) {

            $recordDetailsExpirationTemp = $this->createReportIfBelowThirty($this->giftExpirationProcessedEvents, 
            'giftExpirationEvents');
        }

        // Create a table with the giftPurchasedPendingClaim details
        if ($giftPurchasedPendingClaimCount > 0 && $giftPurchasedPendingClaimCount <= 30) {

            $giftPurchasedPendingClaimTemp = $this->createReportIfBelowThirty($this->giftPurchasedPendingClaim, 
            'inviteSendDate');
        }

        // Create a table with the giftPurchasedDelayedShipment details
        if ($giftPurchasedDelayedShipmentCount > 0 && $giftPurchasedDelayedShipmentCount <= 30) {

            $giftPurchasedDelayedShipmentTemp = $this->createReportIfBelowThirty($this->giftPurchasedDelayedShipment, 
            'gifteeShipmentDate');
        }

        $recipientEmails = 'testing@p80w.com';
        // $recipientEmails = 'mikegordon79@gmail.com';

        // Processing gift event created report
        $subject = "Nurish-Dev Shopify gift pack event creation notification - ".$reportSendingTime;

        $emptyReportStatus = 0;
        if ($gifteeEventCreatedCount == 0 && $giftExpirationEventCreatedCount == 0 && 
        $giftPurchasedPendingClaimCount == 0 && $giftPurchasedDelayedShipmentCount == 0) {

            Log::channel('giftCronLog')->info("No giftee and gift_subscription_expiring events were created in 
            iterable during this execution");
            $commonContent = '<p>No giftee and gift_subscription_expiring events were created in iterable 
            during this execution</p>';
            $emptyReportStatus = 1;
        }

        if ($emptyReportStatus == 0) {

            // Giftee and gift expiration event creation
            if ($gifteeEventCreatedCount == 0 && $giftExpirationEventCreatedCount == 0) {

                Log::channel('giftCronLog')->info("No giftee and gift_subscription_expiring events were created in 
                iterable during this execution");
                $recordDetails .= '<b>Giftee events:</b>';
                $recordDetails .= '<br>No giftee events were created in iterable during this execution';
                $recordDetails .= '<br><br>';
                $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
                $recordDetails .= '<br>No gift_subscription_expiring events were created in iterable during this execution';

            } elseif ($gifteeEventCreatedCount > 0 && $gifteeEventCreatedCount <= 30 && 
            $giftExpirationEventCreatedCount == 0) {

                $commonContent = '<p>Please see the below details for the gift events that were created during this 
                execution.</p>';

                $recordDetails .= '<b>Giftee events:</b>';
                $recordDetails .= $recordDetailsGifteeTemp;
                $recordDetails .= '<br>';
                $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
                $recordDetails .= '<br>No gift_subscription_expiring events were created in iterable during this execution';

            } elseif ($giftExpirationEventCreatedCount > 0 && $giftExpirationEventCreatedCount <= 30 && 
            $gifteeEventCreatedCount == 0) {

                $commonContent = '<p>Please see the below details for the gift events that were created during this 
                execution.</p>';

                $recordDetails .= '<b>Giftee events:</b>';
                $recordDetails .= '<br>No giftee events were created in iterable during this execution';
                $recordDetails .= '<br><br>';
                $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
                $recordDetails .= $recordDetailsExpirationTemp;

            } elseif ($gifteeEventCreatedCount > 30 && $giftExpirationEventCreatedCount > 30) {

                $gifteeFileUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->gifteeEventsCreatedReport;
                $gifteeExpirationFileUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->giftExpirationEventsCreatedReport;
                $commonContent = '<p>Please see the below details for the gift events that were created during this 
                execution.</p>';
                
                $recordDetails .= '<b>Giftee events:</b>';
                $recordDetails .= '<br>Please <a href="'.$gifteeFileUrl.'">click here</a> to download the file.';
                $recordDetails .= '<br><br>';
                $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
                $recordDetails .= '<br>Please <a href="'.$gifteeExpirationFileUrl.'">click here</a> to download the file.';

            } elseif ($gifteeEventCreatedCount > 0 && $gifteeEventCreatedCount <= 30 && 
            $giftExpirationEventCreatedCount > 0 && $giftExpirationEventCreatedCount <= 30) {

                $commonContent = '<p>Please see the below details for the gift events that were created during this 
                execution.</p>';

                $recordDetails .= '<b>Giftee events:</b>';
                $recordDetails .= $recordDetailsGifteeTemp;
                $recordDetails .= '<br>';
                $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
                $recordDetails .= $recordDetailsExpirationTemp;

            } elseif ($gifteeEventCreatedCount == 0 && $giftExpirationEventCreatedCount > 30) {

                $gifteeExpirationFileUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->giftExpirationEventsCreatedReport;
                $commonContent = '<p>Please see the below details for the gift events that were created during this 
                execution.</p>';

                $recordDetails .= '<b>Giftee events:</b>';
                $recordDetails .= '<br>No giftee events were created in iterable during this execution';
                $recordDetails .= '<br><br>';
                $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
                $recordDetails .= '<br>Please <a href="'.$gifteeExpirationFileUrl.'">click here</a> to download the file.';
                
            } elseif ($giftExpirationEventCreatedCount == 0 && $gifteeEventCreatedCount > 30) {

                $gifteeFileUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->gifteeEventsCreatedReport;
                $commonContent = '<p>Please see the below details for the gift events that were created during this 
                execution.</p>';

                $recordDetails .= '<b>Giftee events:</b>';
                $recordDetails .= '<br>Please <a href="'.$gifteeFileUrl.'">click here</a> to download the file.';
                $recordDetails .= '<br><br>';
                $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
                $recordDetails .= '<br>No gift_subscription_expiring events were created in iterable during this execution';

            } elseif ($gifteeEventCreatedCount > 0 && $gifteeEventCreatedCount <= 30 && 
            $giftExpirationEventCreatedCount > 30) {

                $gifteeExpirationFileUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->giftExpirationEventsCreatedReport;
                $commonContent = '<p>Please see the below details for the gift events that were created during this 
                execution.</p>';

                $recordDetails .= '<b>Giftee events:</b>';
                $recordDetails .= $recordDetailsGifteeTemp;
                $recordDetails .= '<br>';
                $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
                $recordDetails .= '<br>Please <a href="'.$gifteeExpirationFileUrl.'">click here</a> to download the file.';

            } elseif ($giftExpirationEventCreatedCount > 0 && $giftExpirationEventCreatedCount <= 30 && 
            $gifteeEventCreatedCount > 30) {

                $gifteeFileUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->gifteeEventsCreatedReport;
                $commonContent = '<p>Please see the below details for the gift events that were created during this 
                execution.</p>';

                $recordDetails .= '<b>Giftee events:</b>';
                $recordDetails .= '<br>Please <a href="'.$gifteeFileUrl.'">click here</a> to download the file.';
                $recordDetails .= '<br><br>';
                $recordDetails .= '<b>Gift_subscription_expiring events:</b>';
                $recordDetails .= $recordDetailsExpirationTemp;
            }


            $recordDetails2 .= '<br><br>';
            // giftPurchasedPendingClaim and giftPurchasedDelayedShipment report creation
            if ($giftPurchasedPendingClaimCount == 0 && $giftPurchasedDelayedShipmentCount == 0) {

                Log::channel('giftCronLog')->info("No giftPurchasedPendingClaim and giftPurchasedDelayedShipment details 
                were found during this execution");
                $recordDetails2 .= '<b>Gift purchased pending claims:</b>';
                $recordDetails2 .= '<br>No gift purchased pending claims were found during this execution';
                $recordDetails2 .= '<br><br>';
                $recordDetails2 .= '<b>Gift purchased pending delayed Shipments:</b>';
                $recordDetails2 .= '<br>No gift purchased pending delayed Shipment were found during this execution';

            } elseif ($giftPurchasedPendingClaimCount > 0 && $giftPurchasedPendingClaimCount <= 30 && 
            $giftPurchasedDelayedShipmentCount == 0) {

                // $commonContent = '<p>Please see the below details for the gift events that were created during this 
                // execution.</p>';

                $recordDetails2 .= '<b>Gift purchased pending claims:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedPendingClaimCount.' gifts purchased pending claim';
                $recordDetails2 .= $giftPurchasedPendingClaimTemp;
                $recordDetails2 .= '<br>';
                $recordDetails2 .= '<b>Gift purchased pending delayed Shipments:</b>';
                $recordDetails2 .= '<br>No gift purchased pending delayed Shipment were found during this execution';

            } elseif ($giftPurchasedDelayedShipmentCount > 0 && $giftPurchasedDelayedShipmentCount <= 30 && 
            $giftPurchasedPendingClaimCount == 0) {

                // $commonContent = '<p>Please see the below details for the gift events that were created during this 
                // execution.</p>';

                $recordDetails2 .= '<b>Gift purchased pending claims:</b>';
                $recordDetails2 .= '<br>No gift purchased pending claims were found during this execution';
                $recordDetails2 .= '<br><br>';
                $recordDetails2 .= '<b>Gift purchased pending delayed Shipments:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedDelayedShipmentCount.' gifts purchased pending 
                delayed Shipment';
                $recordDetails2 .= $giftPurchasedDelayedShipmentTemp;

            } elseif ($giftPurchasedPendingClaimCount > 30 && $giftPurchasedDelayedShipmentCount > 30) {

                $giftPurchasedPendingClaimUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->giftPurchasedPendingClaimReport;
                $giftPurchasedDelayedShipmentUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->giftPurchasedDelayedShipmentReport;
                // $commonContent = '<p>Please see the below details for the gift events that were created during this 
                // execution.</p>';
                
                $recordDetails2 .= '<b>Gift purchased pending claims:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedPendingClaimCount.' gifts purchased pending claim';
                $recordDetails2 .= 'Please <a href="'.$giftPurchasedPendingClaimUrl.'">click here</a> to download the file.';
                $recordDetails2 .= '<br><br>';
                $recordDetails2 .= '<b>Gift purchased pending delayed Shipments:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedDelayedShipmentCount.' gifts purchased pending 
                delayed Shipment';
                $recordDetails2 .= '<br>Please <a href="'.$giftPurchasedDelayedShipmentUrl.'">click here</a> to download the file.';

            } elseif ($giftPurchasedPendingClaimCount > 0 && $giftPurchasedPendingClaimCount <= 30 && 
            $giftPurchasedDelayedShipmentCount > 0 && $giftPurchasedDelayedShipmentCount <= 30) {

                // $commonContent = '<p>Please see the below details for the gift events that were created during this 
                // execution.</p>';

                $recordDetails2 .= '<b>Gift purchased pending claims:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedPendingClaimCount.' gifts purchased pending claim';
                $recordDetails2 .= $giftPurchasedPendingClaimTemp;
                $recordDetails2 .= '<br>';
                $recordDetails2 .= '<b>Gift purchased pending delayed Shipments:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedDelayedShipmentCount.' gifts purchased pending 
                delayed Shipment';
                $recordDetails2 .= $giftPurchasedDelayedShipmentTemp;

            } elseif ($giftPurchasedPendingClaimCount == 0 && $giftPurchasedDelayedShipmentCount > 30) {

                $giftPurchasedDelayedShipmentUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->giftPurchasedDelayedShipmentReport;
                // $commonContent = '<p>Please see the below details for the gift events that were created during this 
                // execution.</p>';

                $recordDetails2 .= '<b>Gift purchased pending claims:</b>';
                $recordDetails2 .= '<br>No gift purchased pending claims were found during this execution';
                $recordDetails2 .= '<br><br>';
                $recordDetails2 .= '<b>Gift purchased pending delayed Shipments:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedDelayedShipmentCount.' gifts purchased pending 
                delayed Shipment';
                $recordDetails2 .= '<br>Please <a href="'.$giftPurchasedDelayedShipmentUrl.'">click here</a> to download the file.';
                
            } elseif ($giftPurchasedDelayedShipmentCount == 0 && $giftPurchasedPendingClaimCount > 30) {

                $giftPurchasedPendingClaimUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->giftPurchasedPendingClaimReport;
                // $commonContent = '<p>Please see the below details for the gift events that were created during this 
                // execution.</p>';

                $recordDetails2 .= '<b>Gift purchased pending claims:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedPendingClaimCount.' gifts purchased pending claim';
                $recordDetails2 .= '<br>Please <a href="'.$giftPurchasedPendingClaimUrl.'">click here</a> to download the file.';
                $recordDetails2 .= '<br><br>';
                $recordDetails2 .= '<b>Gift purchased pending delayed Shipments:</b>';
                $recordDetails2 .= '<br>No gift purchased pending delayed Shipment were found during this execution';

            } elseif ($giftPurchasedPendingClaimCount > 0 && $giftPurchasedPendingClaimCount <= 30 && 
            $giftPurchasedDelayedShipmentCount > 30) {

                $giftPurchasedDelayedShipmentUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->giftPurchasedDelayedShipmentReport;
                // $commonContent = '<p>Please see the below details for the gift events that were created during this 
                // execution.</p>';

                $recordDetails2 .= '<b>Gift purchased pending claims:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedPendingClaimCount.' gifts purchased pending claim';
                $recordDetails2 .= $giftPurchasedPendingClaimTemp;
                $recordDetails2 .= '<br>';
                $recordDetails2 .= '<b>Gift purchased pending delayed Shipments:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedDelayedShipmentCount.' gifts purchased pending 
                delayed Shipment';
                $recordDetails2 .= '<br>Please <a href="'.$giftPurchasedDelayedShipmentUrl.'">click here</a> to download the file.';

            } elseif ($giftPurchasedDelayedShipmentCount > 0 && $giftPurchasedDelayedShipmentCount <= 30 && 
            $giftPurchasedPendingClaimCount > 30) {

                $giftPurchasedPendingClaimUrl = $csvFilePath.'/GiftPackEventCreatedReport/'.$this->reportFileDirectory.'/'.
                $this->giftPurchasedPendingClaimReport;
                // $commonContent = '<p>Please see the below details for the gift events that were created during this 
                // execution.</p>';

                $recordDetails2 .= '<b>Gift purchased pending claims:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedPendingClaimCount.' gifts purchased pending claim';
                $recordDetails2 .= '<br>Please <a href="'.$giftPurchasedPendingClaimUrl.'">click here</a> to download the file.';
                $recordDetails2 .= '<br><br>';
                $recordDetails2 .= '<b>Gift purchased pending delayed Shipments:</b>';
                $recordDetails2 .= '<br>There are currently '.$giftPurchasedDelayedShipmentCount.' gifts purchased pending 
                delayed Shipment';
                $recordDetails2 .= $giftPurchasedDelayedShipmentTemp;
            }
        }

        $allRecordDetails = '';
        $allRecordDetails = $recordDetails.$recordDetails2;
        $iterableMailRequest = array(
            "campaignId" => intval(env('GIFT_PACK_EVENT_CRON_CAMPAIGN_ID', true)),
            "recipientEmail" => $recipientEmails,
            "dataFields" => array(                
                "recordDetails" => $allRecordDetails,
                "subject" => $subject,
                "commonContent" => $commonContent
            ),
        );
        
        if(!empty($iterableMailRequest)) {
            
            $mailTriggerResponse = $this->iterableObj->sendMailFromIterable(json_encode($iterableMailRequest));

            if (strtolower(trim($mailTriggerResponse->code)) == 'success') {           
                
                Log::channel('giftCronLog')->info("giftee event created report mail sent");
    
            } else {
                
                Log::channel('giftCronLog')->info('giftee event created report mail not sent');
                Log::channel('giftCronLog')->info(json_encode($mailTriggerResponse));
                $error = !empty($mailTriggerResponse->code) ? $mailTriggerResponse->code : '';
    
                if (empty($error)) {
    
                    $returnData = [
                        'error' => 'Failed to send mail(' . $recipientEmails . ')',
                    ];
                } else {
    
                    $returnData = [
                        'error' => $error . ' (' . $recipientEmails . ')',
                    ];
                }
                Log::channel('giftCronLog')->info($returnData['error']);
            }
        }
    }

    /**
     * Create report if below 30
     */
    public function createReportIfBelowThirty($giftData, $type = NULL) {

        $columnHeading = 'Status';
        $columnValue = 'Status';
        $iterableUserProfileLink = '';
        if (!empty($type)) {

           if ($type == 'inviteSendDate') {

            $columnValue = 'inviteSendDate';
            $columnHeading = 'InviteSendDate';
           } else if ($type == 'gifteeShipmentDate') {

            $columnValue = 'gifteeShipmentDate';
            $columnHeading = 'GifteeShipmentDate';
           }

           if ($type == 'gifteeEvents') {

            $iterableUserProfileLink = env("ITERABLE_USER_PROFILE_LINK", true);
           }
        }

        $recordDetailsTemp = '';
        if (!empty($giftData)) {           
            $recordDetailsTemp .= "
            <table>
                <tr>
                    <td>#</td>                
                    <td>GifteeName</td>
                    <td>GifteeEmail</td> 
                    <td>GifteeShopifyCustomerId</td> 
                    <td>PurchaserShopifyCustomerId</td>
                    <td>".$columnHeading."</td>                       
                </tr>";

            $i = 1;
            foreach ($giftData as $record) {
                        
                $emailAddressSection = '';
                if (!empty($iterableUserProfileLink)) {

                    $emailAddressSection = "<a href='".$iterableUserProfileLink.$record['gifteeEmail']."' target='_blank'>".$record['gifteeEmail']."</a>";
                } else {

                    $emailAddressSection = $record['gifteeEmail'];
                }
                $recordDetailsTemp .= "
                <tr>
                    <td>".$i."</td>                             
                    <td>".$record['gifteeName']."</td> 
                    <td>".$emailAddressSection."</td>  
                    <td><a href='".$this->shopifyCustomerLink.$record['gifteeShopifyCustomerId']."' target='_blank'>".$record['gifteeShopifyCustomerId']."</a></td>
                    <td><a href='".$this->shopifyCustomerLink.$record['purchaserShopifyCustomerId']."' target='_blank'>".$record['purchaserShopifyCustomerId']."</a></td>
                    <td>".$record[$columnValue]."</td>
                </tr>";
                $i++;            
            }
            $recordDetailsTemp .= '</table>';

            return $recordDetailsTemp;
        }
    }

    /**
     * Create report if above 30
     */
    public function createReportIfAboveThirty($giftData, $csvFilename, $type = NULL) {

        $fileGift   = $this->logFiles[$csvFilename]['filename'];
        $filePointerGift = fopen($fileGift, "a");

        $columnValue = 'Status';
        if (!empty($type)) {

           if ($type == 'inviteSendDate') {

            $columnValue = 'inviteSendDate';
           } else {

            $columnValue = 'gifteeShipmentDate';
           }
        }

        foreach ($giftData as $giftKey => $giftValue) {

            $precessedGift = [];
            $precessedGift = array(
                $giftValue['gifteeName'],
                $giftValue['gifteeEmail'],
                $giftValue['gifteeShopifyCustomerId'],
                $giftValue['purchaserShopifyCustomerId'],  
                $giftValue[$columnValue]          
            );
            fputcsv($filePointerGift, $precessedGift);
        }
    }

    /**
     * Endpoint to pause the gift subscription by the gift recipient
     */
    public function gifteePauseSubscription(Request $request) {

        $requestData = $request->input('data');

        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            Log::channel('giftPackLog')->info("gifteePauseSubscription: Invalid endpoint request");
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $receivedData = json_decode($requestData);

        if (empty($receivedData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            Log::channel('giftPackLog')->info("gifteePauseSubscription: Invalid endpoint request, endpoint request:".
            json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }
        $emptyParameters = [];
        $requiredRequestKeys = ["subscription_id", "email_address", "paused_period"];
        $endpointRequestKeys = [];
        $keyComparedResult = [];

        foreach ($receivedData as $key => $receiveValue) {

            $requestValues = '';
            $endpointRequestKeys[] = trim($key);
            $requestValues = !empty($receiveValue) ? trim($receiveValue) : '';
            if (empty($requestValues)) {
                $emptyParameters[] = $key;
            }
        }

        $keyComparedResult = array_diff($requiredRequestKeys, $endpointRequestKeys);
        if(!empty($keyComparedResult)) {

            $errorMessage = '';
            $errorMessage = (count($keyComparedResult) > 1) ? 
            "Missing required parameters(".implode(",", $keyComparedResult).")" :
            "Required parameter ".implode(",", $keyComparedResult)." is missing";
            $returnData = [
                'error' => $errorMessage
            ];            
            Log::channel('giftPackLog')->info("gifteePauseSubscription: Missing required parameters(".
            implode(",", $keyComparedResult)."), endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        if (!empty($emptyParameters)) {

            $errorMessage = '';
            $errorMessage = (count($emptyParameters) > 1) ? 
            "Found empty values in the request(".implode(",", $emptyParameters).")" :
            "The ".implode(",", $emptyParameters)." value is empty";
            $returnData = [
                'error' => $errorMessage
            ];            
            Log::channel('giftPackLog')->info("gifteePauseSubscription: Found empty values in the request(".
            implode(",", $emptyParameters)."), endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $subscriptionId = !empty($receivedData->subscription_id) ? trim($receivedData->subscription_id) : '';
        $emailAddress = !empty($receivedData->email_address) ? trim($receivedData->email_address) : '';
        $pausedPeriod = !empty($receivedData->paused_period) ? trim($receivedData->paused_period) : '';

        if (empty($subscriptionId) || empty($emailAddress) || empty($pausedPeriod)) {

            $returnData = [
                'error' => "Some parameters are missing in the request"
            ];            
            Log::channel('giftPackLog')->info("gifteePauseSubscription: Some parameters are missing in the request, 
            endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $subscriptionDetails =  $this->rechargeObj->getSubscription($subscriptionId);

        if (!empty($subscriptionDetails->error) || !empty($subscriptionDetails->errors) || empty($subscriptionDetails)) {

            $rechargeError  = "Unknown error";
            if (!empty($subscriptionDetails->error)) {

                $rechargeError = json_encode($subscriptionDetails->error);
            } else if (!empty($subscriptionDetails->errors)) {

                $rechargeError = json_encode($subscriptionDetails->errors);
            } else if (empty($subscriptionDetails)) {

                $rechargeError = 'Failed to get the subscription details by subscriptionId';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("gifteePauseSubscription: Failed to get the subscription 
            details by subscriptionId(subscriptionId:".$subscriptionId."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $gifteeEmailAddressInSubscriptionLip = '';
        if (empty($subscriptionDetails->subscription->properties)) {

            $returnData = [
                'error' => "Failed to find the provided giftee email address in the subscription LIP"
            ];

            Log::channel('giftPackLog')->info("gifteePauseSubscription: Failed to find the provided giftee email 
            address in the subscription LIP(subscriptionId:".$subscriptionId.")");
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        foreach ($subscriptionDetails->subscription->properties as $lipKey => $lipValue) {

            if (trim($lipValue->name) == 'giftee_email') {

                $gifteeEmailAddressInSubscriptionLip = trim($lipValue->value);
            }
        }

        if (strtolower(trim($emailAddress)) != strtolower(trim($gifteeEmailAddressInSubscriptionLip))) {

            $returnData = [
                'error' => "Failed to match the requested email address with the email address in the subscription LIP"
            ];

            Log::channel('giftPackLog')->info("gifteePauseSubscription: Failed to match the requested email address 
            with the email address in the subscription LIP(subscriptionId:".$subscriptionId."), 
            endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        // if (empty($subscriptionDetails->subscription->next_charge_scheduled_at)) {

        //     $returnData = [
        //         'error' => "The next charge date is not found in the subscription"
        //     ];

        //     Log::channel('giftPackLog')->info("gifteePauseSubscription: The next charge date is not found in the 
        //     subscription(subscriptionId:".$subscriptionId.")");
        //     return $this->returnData(600, "error", $returnData, $request->input('callback'));
        // }
        $subscriptionAddressId = !empty($subscriptionDetails->subscription->address_id) ?  
        trim($subscriptionDetails->subscription->address_id): '';

        if (empty($subscriptionAddressId)) {

                $returnData = [
                'error' => "The address_id not found in the subscription"
            ];

            Log::channel('giftPackLog')->info("gifteePauseSubscription: The address_id not found in the 
            subscription(subscriptionId:".$subscriptionId.")");
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        // Get all order details from recharge by address ID
        $allOrderDetails = $this->rechargeObj->getRechargeOrdersByAddressId($subscriptionAddressId);
        if (!empty($allOrderDetails->error) || !empty($allOrderDetails->errors) ||
            empty($allOrderDetails) || empty($allOrderDetails->orders)) {

            $rechargeError  = "Unknown error";
            if (!empty($allOrderDetails->error)) {

                $rechargeError = json_encode($allOrderDetails->error);
            } else if (!empty($allOrderDetails->errors)) {

                $rechargeError = json_encode($allOrderDetails->errors);
            } else if (empty($allOrderDetails) || empty($allOrderDetails->orders)) {

                $rechargeError = 'Failed to get the order details from recharge';
            }

            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("gifteePauseSubscription: Failed to get the order details by using 
            the given addressId: ".$subscriptionAddressId."), endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $nextOrderScheduledDate = '';
        $queuedOrderId = '';
        foreach (array_reverse($allOrderDetails->orders) as $orderKey => $orderValue) {

            $orderStatus = !empty($orderValue->status) ? strtolower(trim($orderValue->status)) : '';
            if ($orderStatus == 'queued') {

                $nextOrderScheduledDate = $orderValue->scheduled_at;
                $queuedOrderId = $orderValue->id;
                break;
            }
        }

        if (empty($nextOrderScheduledDate)) {

            $returnData = [
                'error' => "Failed to find the upcoming queued order"
            ];

            Log::channel('giftPackLog')->info("gifteePauseSubscription: Failed to find the upcoming queued order, 
            endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $nextOrderScheduledDate = date('Y-m-d', strtotime($nextOrderScheduledDate));
        $pausePeriod = (int)$pausedPeriod;
        $newOrderScheduledDate = date('Y-m-d',strtotime('+'.$pausePeriod.' days',strtotime($nextOrderScheduledDate)));

        $queuedOrderShipmentDate = $newOrderScheduledDate;
        foreach (array_reverse($allOrderDetails->orders) as $orderKey => $orderValue) {

            if (trim(strtolower($orderValue->status)) == 'queued') {

                $queuedOrderId = '';
                $nextOrderScheduledDate = '';
                $newOrderScheduledDate = '';
                $payload = [];
                $queuedOrderId = trim($orderValue->id);

                $newOrderScheduledDate = $queuedOrderShipmentDate;
                $payload = array(
                    "scheduled_at" => $newOrderScheduledDate
                );
                sleep(1);
                $queuedOrderUpdateResponse = $this->rechargeObj->updateOrderScheduledDate($queuedOrderId, json_encode($payload));

                if (!empty($queuedOrderUpdateResponse->error) || !empty($queuedOrderUpdateResponse->errors) ||
                empty($queuedOrderUpdateResponse)) {

                    $rechargeError  = "Unknown error";
                    if (!empty($queuedOrderUpdateResponse->error)) {

                        $rechargeError = json_encode($queuedOrderUpdateResponse->error);
                    } else if (!empty($queuedOrderUpdateResponse->errors)) {

                        $rechargeError = json_encode($queuedOrderUpdateResponse->errors);
                    } else if (empty($queuedOrderUpdateResponse)) {

                        $rechargeError = 'Failed to update the order scheduled_at date';
                    }

                    $returnData = [
                        'error' => $rechargeError
                    ];

                    Log::channel('giftPackLog')->info("gifteePauseSubscription: Failed to update the order scheduled_at 
                    date(subscriptionId:".$subscriptionId.", orderId: ".$queuedOrderId.")");
                    return $this->returnData(600, "error", $returnData, $request->input('callback'));
                }
                $nextOrderScheduledDate = date('Y-m-d', strtotime($queuedOrderShipmentDate));
                $orderShippingPeriodDifference = 28;
                $orderShippingPeriodDifference = (int)$orderShippingPeriodDifference;
                $newOrderScheduledDate = date('Y-m-d',strtotime('+'.$orderShippingPeriodDifference.' days',
                strtotime($nextOrderScheduledDate)));
                $queuedOrderShipmentDate = $newOrderScheduledDate;
            }
        }

        $returnData = [
            'message' => "Successfully paused the subscription"
        ];

        Log::channel('giftPackLog')->info("gifteePauseSubscription: Successfully paused the subscription
        (subscriptionId:".$subscriptionId.")");
        return $this->returnData(200, "success", $returnData, $request->input('callback'));
    }

    /**
     * Endpoint to cancel the gift subscription by the gift recipient
     */
    public function gifteeCancelSubscription(Request $request) {

        $requestData = $request->input('data');

        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            Log::channel('giftPackLog')->info("gifteeCancelSubscription: Invalid endpoint request");
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $receivedData = json_decode($requestData);

        if (empty($receivedData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            Log::channel('giftPackLog')->info("gifteeCancelSubscription: Invalid endpoint request, endpoint request:".
            json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $subscriptionId = !empty($receivedData->subscription_id) ? trim($receivedData->subscription_id) : '';
        $emailAddress = !empty($receivedData->email_address) ? trim($receivedData->email_address) : '';

        if (empty($subscriptionId)) {

            $returnData = [
                'error' => 'Required parameter subscription_id or its value is missing',
            ];            
            Log::channel('giftPackLog')->info("gifteeCancelSubscription: Required parameter subscription_id or 
            its value is missing, endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        if (empty($emailAddress)) {

            $returnData = [
                'error' => 'Required parameter email_address or its value is missing',
            ];            
            Log::channel('giftPackLog')->info("gifteeCancelSubscription: Required parameter email_address or 
            its value is missing, endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $subscriptionDetails =  $this->rechargeObj->getSubscription($subscriptionId);

        if (!empty($subscriptionDetails->error) || !empty($subscriptionDetails->errors) || empty($subscriptionDetails)) {

            $rechargeError  = "Unknown error";
            if (!empty($subscriptionDetails->error)) {

                $rechargeError = json_encode($subscriptionDetails->error);
            } else if (!empty($subscriptionDetails->errors)) {

                $rechargeError = json_encode($subscriptionDetails->errors);
            } else if (empty($subscriptionDetails)) {

                $rechargeError = 'Failed to get the subscription details by subscriptionId';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("gifteeCancelSubscription: Failed to get the subscription 
            details by subscriptionId(subscriptionId:".$subscriptionId."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $gifteeEmailAddressInSubscriptionLip = '';
        if (empty($subscriptionDetails->subscription->properties)) {

            $returnData = [
                'error' => "Failed to find the provided giftee email address in the subscription LIP"
            ];

            Log::channel('giftPackLog')->info("gifteeCancelSubscription: Failed to find the provided giftee email 
            address in the subscription LIP(subscriptionId:".$subscriptionId.")");
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        foreach ($subscriptionDetails->subscription->properties as $lipKey => $lipValue) {

            if (trim($lipValue->name) == 'giftee_email') {

                $gifteeEmailAddressInSubscriptionLip = trim($lipValue->value);
            }
        }

        if (strtolower(trim($emailAddress)) != strtolower(trim($gifteeEmailAddressInSubscriptionLip))) {

            $returnData = [
                'error' => "Failed to match the requested email address with the email address in the subscription LIP"
            ];

            Log::channel('giftPackLog')->info("gifteeCancelSubscription: Failed to match the requested email address 
            with the email address in the subscription LIP(subscriptionId:".$subscriptionId."), 
            endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $cancelPayload = array(
            "cancellation_reason" => "other reason"
        );
        
        $subscriptionCancelResponse = $this->rechargeObj->cancelSubscription($subscriptionId, json_encode($cancelPayload));
        if (!empty($subscriptionCancelResponse->error) || !empty($subscriptionCancelResponse->errors) ||
        empty($subscriptionCancelResponse)) {

            $rechargeError  = "Unknown error";
            if (!empty($subscriptionCancelResponse->error)) {

                $rechargeError = json_encode($subscriptionCancelResponse->error);
            } else if (!empty($subscriptionCancelResponse->errors)) {

                $rechargeError = json_encode($subscriptionCancelResponse->errors);
            } else if (empty($subscriptionCancelResponse)) {

                $rechargeError = 'Failed to cancel the subscription';
            }

            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("gifteeCancelSubscription: Failed to cancel the subscription
            (subscriptionId:".$subscriptionId.")");
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $returnData = [
            'message' => "Successfully canceled the subscription"
        ];

        Log::channel('giftPackLog')->info("gifteeCancelSubscription: Successfully canceled the subscription
        (subscriptionId:".$subscriptionId.")");
        return $this->returnData(200, "success", $returnData, $request->input('callback'));
    }

    /**
     * Endpoint to update the gift subscription address by the gift recipient
     */
    public function gifteeSubscriptionAddressUpdate(Request $request) {

        $requestData = $request->input('data');

        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            Log::channel('giftPackLog')->info("gifteeSubscriptionAddressUpdate: Invalid endpoint request");
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $receivedData = json_decode($requestData);

        if (empty($receivedData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            Log::channel('giftPackLog')->info("gifteeSubscriptionAddressUpdate: Invalid endpoint request, 
            endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $subscriptionId = !empty($receivedData->subscription_id) ? trim($receivedData->subscription_id) : '';
        $addressId = !empty($receivedData->address_id) ? trim($receivedData->address_id) : '';
        $emailAddress = !empty($receivedData->email_address) ? trim($receivedData->email_address) : '';

        if (empty($subscriptionId)) {

            $returnData = [
                'error' => 'Required parameter subscription_id or its value is missing',
            ];            
            Log::channel('giftPackLog')->info("gifteeSubscriptionAddressUpdate: Required parameter subscription_id or 
            its value is missing, endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        if (empty($addressId)) {

            $returnData = [
                'error' => 'Required parameter address_id or its value is missing',
            ];            
            Log::channel('giftPackLog')->info("gifteeSubscriptionAddressUpdate: Required parameter address_id or 
            its value is missing, endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        if (empty($emailAddress)) {

            $returnData = [
                'error' => 'Required parameter email_address or its value is missing',
            ];            
            Log::channel('giftPackLog')->info("gifteeSubscriptionAddressUpdate: Required parameter email_address or 
            its value is missing, endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $subscriptionDetails =  $this->rechargeObj->getSubscription($subscriptionId);

        if (!empty($subscriptionDetails->error) || !empty($subscriptionDetails->errors) || empty($subscriptionDetails)) {

            $rechargeError  = "Unknown error";
            if (!empty($subscriptionDetails->error)) {

                $rechargeError = json_encode($subscriptionDetails->error);
            } else if (!empty($subscriptionDetails->errors)) {

                $rechargeError = json_encode($subscriptionDetails->errors);
            } else if (empty($subscriptionDetails)) {

                $rechargeError = 'Failed to get the subscription details by subscriptionId';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("gifteeSubscriptionAddressUpdate: Failed to get the subscription 
            details by subscriptionId(subscriptionId:".$subscriptionId."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $gifteeEmailAddressInSubscriptionLip = '';
        if (empty($subscriptionDetails->subscription->properties)) {

            $returnData = [
                'error' => "Failed to find the provided giftee email address in the subscription LIP"
            ];

            Log::channel('giftPackLog')->info("gifteeSubscriptionAddressUpdate: Failed to find the provided giftee email 
            address in the subscription LIP(subscriptionId:".$subscriptionId.")");
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        foreach ($subscriptionDetails->subscription->properties as $lipKey => $lipValue) {

            if (trim($lipValue->name) == 'giftee_email') {

                $gifteeEmailAddressInSubscriptionLip = trim($lipValue->value);
            }
        }

        if (strtolower(trim($emailAddress)) != strtolower(trim($gifteeEmailAddressInSubscriptionLip))) {

            $returnData = [
                'error' => "Failed to match the requested email address with the email address in the subscription LIP"
            ];

            Log::channel('giftPackLog')->info("gifteeSubscriptionAddressUpdate: Failed to match the requested email address 
            with the email address in the subscription LIP(subscriptionId:".$subscriptionId."), 
            endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $subscriptionAddressId = !empty($subscriptionDetails->subscription->address_id) ? 
        trim($subscriptionDetails->subscription->address_id) : '';

        if ($addressId != $subscriptionAddressId) {

            $returnData = [
                'error' => 'Failed to find the provided address_id in the gift subscription',
            ];            
            Log::channel('giftPackLog')->info("gifteeSubscriptionAddressUpdate: Failed to find the provided address_id 
            in the gift subscription, endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $addressUpdateRequest = [];
        $addressUpdateRequest = array(
            "first_name" => !empty($receivedData->first_name) ? trim($receivedData->first_name) : '',
            "last_name"  => !empty($receivedData->last_name) ? trim($receivedData->last_name) : '',
            "address1"   => !empty($receivedData->address1) ? trim($receivedData->address1) : '',
            "address2"   => !empty($receivedData->address2) ? trim($receivedData->address2) : '',
            "company"    => !empty($receivedData->company) ? trim($receivedData->company) : '',
            "city"       => !empty($receivedData->city) ? trim($receivedData->city) : '',
            "country"    => !empty($receivedData->country) ? trim($receivedData->country) : '',
            "zip"        => !empty($receivedData->zip) ? trim($receivedData->zip) : '',
            "province"   => !empty($receivedData->province) ? trim($receivedData->province) : '',
            "phone"      => !empty($receivedData->phone) ? trim($receivedData->phone) : ''
        );

        $addressUpdateResponse = $this->rechargeObj->updateSubscriptionAddress($addressId, json_encode($addressUpdateRequest));
        if (!empty($addressUpdateResponse->error) || !empty($addressUpdateResponse->errors) || 
        empty($addressUpdateResponse)) {

            $rechargeError  = "Unknown error";
            if (!empty($addressUpdateResponse->error)) {

                $rechargeError = json_encode($addressUpdateResponse->error);
            } else if (!empty($addressUpdateResponse->errors)) {

                $rechargeError = json_encode($addressUpdateResponse->errors);
            } else if (empty($addressUpdateResponse)) {

                $rechargeError = 'Failed to update the subscription address';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftPackLog')->info("gifteeSubscriptionAddressUpdate: Failed to update the subscription 
            address(subscriptionId:".$subscriptionId."), Error: (".$rechargeError."), 
            endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData, $request->input('callback'));
        }

        $returnData = [
            'message' => "Successfully updated the subscription address"
        ];

        Log::channel('giftPackLog')->info("gifteeSubscriptionAddressUpdate: Successfully updated the subscription address
        (subscriptionId:".$subscriptionId.")");
        return $this->returnData(200, "success", $returnData, $request->input('callback'));
    }

    /**
     * Create random password
     */
    public function createRandomPassword() {

        $randomPassword = '';
        $alphabet = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';
        $password = array();
        $alphaLength = strlen($alphabet) - 1;
        for ($i = 0; $i < 16; $i++) {
            $n = rand(0, $alphaLength);
            $password[] = $alphabet[$n];
        }
        $randomPassword = implode($password);
        return $randomPassword;
    }

    /**
     * Function to create the checkout of the gift recipient
     */
    public function createGiftRecipientCheckout(Request $request) {

        $requestData = $request->input('data');
        $CheckoutUrl = env('NURISH_CHECKOUT_URL', true);
        $shopifyRegularPackProductId = env('REGULAR_SUBSCRIPTION_PRODUCT_ID', true);
        $shopifyRegularPackVariantId = env('REGULAR_SUBSCRIPTION_VARIANT_ID', true);
        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];
            Log::channel('giftPackLog')->info("createGiftRecipientCheckout: Invalid endpoint request");            
            return $this->returnData(600, "error", $returnData);
        }

        $receivedData = json_decode($requestData);

        if (empty($receivedData->subscriptionId)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];    
            Log::channel('giftPackLog')->info("createGiftRecipientCheckout: Subscription Id is empty in 
            the endpoint request");        
            return $this->returnData(600, "error", $returnData);
        }
        // Decode endpoint request
        $purchaserSubscriptionId = $this->decodeEndpointRequest($receivedData->subscriptionId);

        $productDetails = $this->getShopifyProductDetails();

        $subscriptionId = !empty($purchaserSubscriptionId) ? trim($purchaserSubscriptionId) : '';
        $giftSubscriptions = $this->rechargeObj->getSubscription($subscriptionId);
        if (!empty($giftSubscriptions->error) || !empty($giftSubscriptions->errors) || empty($giftSubscriptions)) {

            $rechargeError  = "Unknown error";
            if (!empty($giftSubscriptions->error)) {

                $rechargeError = json_encode($giftSubscriptions->error);
            } else if (!empty($giftSubscriptions->errors)) {

                $rechargeError = json_encode($giftSubscriptions->errors);
            } else if (empty($giftSubscriptions)) {

                $rechargeError = 'Failed to get the subscription details by subscriptionId';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('gifteeCheckoutLog')->info("createGiftRecipientCheckout: Failed to get the subscription 
            details by subscriptionId(subscriptionId:".$subscriptionId."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData);
        }
        $giftSubscriptionDetails = '';
        $giftSubscriptionDetails = $giftSubscriptions->subscription;

        if (empty($giftSubscriptionDetails)) {

            $returnData = [
                'error' => 'No subscription found'
            ];
            Log::channel('gifteeCheckoutLog')->info("createGiftRecipientCheckout: No subscription found 
            (SubscriptionId:".$subscriptionId.")");
            return $this->returnData(600, "error", $returnData);
        }

        if (empty($giftSubscriptionDetails->properties)) {
            $returnData = [
                'error' => 'No line item properties are found in the subscription'
            ];
            Log::channel('gifteeCheckoutLog')->info("createGiftRecipientCheckout: No line item properties are 
            found in the subscription (subscriptionId:".$subscriptionId.")");
            return $this->returnData(600, "error", $returnData);
        }
       
        $properties = $giftSubscriptionDetails->properties;
        $requestProperties = [];
        $packs = "";
        $chargeIntervalFrequency = "";
        $gifteeEmail = "";
        $gifteeName = "";

        foreach ($properties as $property) {
            
            if (strpos($property->name, 'Vitamin') !== false) {
                $value = explode('|', $property->value);
                $variantId = trim($value[3]);
                $requestProperties[] = array(
                    'name' => $property->name,
                    'value' => trim($value[0]),
                    'product_id' => array_key_exists($variantId, $productDetails) ? 
                    $productDetails[$variantId]['product_id'] : '',
                    'variant_id' => $variantId,
                    'quantity' => trim($value[2]),
                    'dosage' => trim($value[1])
                );
            }

            if (strpos($property->name, 'Packs') !== false) {
                $packs = $property->value;
            }
            if (strpos($property->name, 'charge_interval_frequency') !== false) {
                $chargeIntervalFrequency = $property->value;
            }
            if (strpos($property->name, 'giftee_email') !== false) {
                $gifteeEmail = $property->value;
            }
            if (strpos($property->name, 'giftee_first_name') !== false) {
                $gifteeName = $property->value;
            }
        }

        $checkoutRequest['checkout'] = array(
            'discount_code' => '',
            'line_items' => array(
                array(
                    'properties' => $requestProperties,
                    'quantity' => !empty($giftSubscriptionDetails->quantity) ? 
                    trim($giftSubscriptionDetails->quantity) : '',
                    'product_id' => $shopifyRegularPackProductId,
                    'variant_id' => $shopifyRegularPackVariantId,
                    'charge_interval_frequency' => $chargeIntervalFrequency,
                    'order_interval_frequency' => !empty($giftSubscriptionDetails->order_interval_frequency) ? 
                    trim($giftSubscriptionDetails->order_interval_frequency) : '',
                    'order_interval_unit' => !empty($giftSubscriptionDetails->order_interval_unit) ? 
                    trim($giftSubscriptionDetails->order_interval_unit) : '',
                    'Packs' => $packs
                )
            ),
            'email' => $gifteeEmail,
            'name' => $gifteeName
        );
        $method = 'GET';
        $checkoutResponse = $this->rechargeObj->callEndpointToCreateCheckout($method,$CheckoutUrl,json_encode($checkoutRequest));
        $checkoutResponse = trim($checkoutResponse,")");
        $checkoutResponse = ltrim($checkoutResponse,"create(");
        $checkoutResponse = json_decode($checkoutResponse,true);

        if (!empty($checkoutResponse['error']) || !empty($checkoutResponse['errors']) || empty($checkoutResponse)) {

            $checkoutError  = "Unknown error";
            if (!empty($checkoutResponse['error'])) {

                $checkoutError = json_encode($checkoutResponse['error']);
            } else if (!empty($checkoutResponse['errors'])) {

                $checkoutError = json_encode($checkoutResponse['errors']);
            } else if (empty($checkoutResponse)) {

                $checkoutError = 'Failed to create the checkout';
            }
            $returnData = [
                'error' => $checkoutError
            ];

            Log::channel('gifteeCheckoutLog')->info("createGiftRecipientCheckout: Failed to create the checkout for the 
            request ".json_encode($checkoutRequest));
            return $this->returnData(600, "error", $returnData);
        }

        if (empty($checkoutResponse['url']) && empty($checkoutResponse)) {

            $returnData = [
                'error' => 'Failed to create the checkout'
            ];
            Log::channel('gifteeCheckoutLog')->info("createGiftRecipientCheckout: Failed to create the checkout for the 
            request ".json_encode($checkoutRequest));
            return $this->returnData(600, "error", $returnData);
        }
        return $this->returnData(200, "success", $checkoutResponse);
    }

    /**
     * Decode endpoint request
     */
    public function decodeEndpointRequest($endPointParameter) {

        $decodedValue = '';
        $decodedValue = $endPointParameter;
        for ($i = 0; $i < 5; $i++) { 

            $decodedValue = base64_decode($decodedValue);
        }
        return $decodedValue;
    }

    /**
     * Function to get all shopify products
    */
    public function getShopifyProductDetails() {

        $shopifyProducts  = [];
        $pageNo = 1;
        $limit  = 200;
        do {
            try {
                // $this->helperObj->printMe("Read Shopify product page count - ". $pageNo);
                // echo '<br>';
                if(!empty($products['link'])) {

                    $products = $this->productObj->getShopifyAllProducts($limit,$products['link']);

                 } else {

                    $products = $this->productObj->getShopifyAllProducts($limit);
                    
                }
            }
            catch (Exception $e){

                echo PHP_EOL."Error form shopify when fetching the
                            product details and error is -  ".$e->getMessage();
                die;
            }

            $productsToSkip = array('4359176454226', '4359186350162', '4426715758674', '4716581486674', '4716582797394'); // Dev store
            //$productsToSkip = array('1809168695367', '4778895736903', '4778912219207', '4780554256455'); // Production
            
            if (!empty($products) && isset($products)) {
                
                foreach ($products['body'] as $product) {

                    if (in_array($product['id'], $productsToSkip)) {
                        continue;
                    }

                    if (!empty($product['variants'])) {

                        $variantDetails = [];
                        foreach ($product['variants'] as $variant) {

                            if (!empty($variant['id'])) {
                                $variantDetails = [
                                    'variant_id' => $variant['id'],
                                    'product_id' => $variant['product_id'],
                                ];
                                $shopifyProducts[$variant['id']] = $variantDetails;
                            }
                        }
                    }
                }
                $productCount = count($products['body']);
            }

            $pageNo++;

            if (empty($products['link'])) {
                break;
            }
        } while($productCount == $limit);

        return $shopifyProducts;
    }

    public function setPaths() {

        $this->logDirectory = dirname(__FILE__) . "/../../../public/GiftPackEventCreatedReport/";
    }

    public function getLogFilesList() {
        
        $this->reportFileDirectory = date('dmY');

        $this->logPath = $this->logDirectory . $this->reportFileDirectory ;
        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);
        }
        
        $this->gifteeEventsCreatedReport = "giftee-event-processed-list-".date("Y_m_d_H_i").".csv";
        $this->giftExpirationEventsCreatedReport = "gift-expiration-event-processed-list-".date("Y_m_d_H_i").".csv";
        $this->giftPurchasedPendingClaimReport = "gift-purchased-pending-claim-list-".date("Y_m_d_H_i").".csv";
        $this->giftPurchasedDelayedShipmentReport = "gift-purchased-delayed-shipment-list-".date("Y_m_d_H_i").".csv";

        $this->logFiles = array(
            'gifteeEventProcessed' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->gifteeEventsCreatedReport,
                "mode"     => "",
                "columns"  => array(
                    "GifteeName",
                    "GifteeEmail",
                    "GifteeShopifyCustomerId",
                    "PurchaserShopifyCustomerId",                   
                    "Status"
                )
            ),
            'giftExpireEventProcessed' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->giftExpirationEventsCreatedReport,
                "mode"     => "",
                "columns"  => array(
                    "GifteeName",
                    "GifteeEmail",
                    "GifteeShopifyCustomerId",
                    "PurchaserShopifyCustomerId",                          
                    "Status"
                )
            ),
            'giftPendingClaimProcessed' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->giftPurchasedPendingClaimReport,
                "mode"     => "",
                "columns"  => array(
                    "GifteeName",
                    "GifteeEmail",
                    "GifteeShopifyCustomerId",
                    "PurchaserShopifyCustomerId",                          
                    "InviteSendDate"
                )
            ),
            'giftDelayedShipmentProcessed' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/".$this->giftPurchasedDelayedShipmentReport,
                "mode"     => "",
                "columns"  => array(
                    "GifteeName",
                    "GifteeEmail",
                    "GifteeShopifyCustomerId",
                    "PurchaserShopifyCustomerId",                          
                    "GifteeShipmentDate"
                )
            )       
        );
    }

    public function setWithLogFiles() {

        if (!empty($this->logFiles)) {

            foreach ($this->logFiles as $logType=>$fileDetails){

                if ($fileDetails['type']=="csv"){
                    $mode = !empty($fileDetails['mode']) ? $fileDetails['mode'] : "w";

                    if ($mode != "r"){
                        $fileName    = $fileDetails['filename'];
                        $filePointer = fopen($fileName,$mode);
                        $columns     = $fileDetails['columns'];
                        fputcsv($filePointer,$columns);
                        chmod($fileName, 0777);
                        fclose($filePointer);
                    }
                }
            }
        }
    }

    /**
     * Endpoint to check the gift recipient password reset status
     */
    public function getGiftRecipientPasswordResetStatus(Request $request) {

        $requestData = $request->input('data');
        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request'
            ];            
            Log::channel('giftRecipientPasswordResetLog')->info("getGiftRecipientPasswordResetStatus: Invalid 
            endpoint request");
            return $this->returnData(600, "error", $returnData);
        }
        $receivedData = json_decode($requestData);
        
        $giftToken = !empty($receivedData->token) ? trim($receivedData->token) : '';
        $emailAddress = !empty($receivedData->email) ? trim($receivedData->email) : '';

        if (empty($giftToken)) {

            $returnData = [
                'error' => 'Required parameter token is missing in the endpoint request'
            ];            
            Log::channel('giftRecipientPasswordResetLog')->info("getGiftRecipientPasswordResetStatus: Required 
            parameter token is missing in the endpoint request, endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        if (empty($emailAddress)) {

            $returnData = [
                'error' => 'Required parameter email address is missing in the endpoint request'
            ];            
            Log::channel('giftRecipientPasswordResetLog')->info("getGiftRecipientPasswordResetStatus: Required 
            parameter email address is missing in the endpoint request, endpoint request:".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        $purchaserSubscriptionId = '';
        $recipientShopifyCustomerId = '';
        $decodedToken = $this->decodeEndpointRequest($giftToken);

        if (strpos($decodedToken, ':') !== false) {

            $explodeToken = explode(":", $decodedToken);
            $purchaserSubscriptionId = $explodeToken[0];
            $recipientShopifyCustomerId = $explodeToken[1];
        }

        if (empty($purchaserSubscriptionId) || empty($recipientShopifyCustomerId)) {

            $returnData = [
                'error' => 'Failed to find the required details from the token',
            ];
            Log::channel('giftRecipientPasswordResetLog')->info("getGiftRecipientPasswordResetStatus: Failed to 
            find the required details from the token, endpoint request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        // Get charge details from recharge by subscriptionId
        $chargeDetails = $this->rechargeObj->getChargeDetailsBySubscriptionId($purchaserSubscriptionId);
        if (!empty($chargeDetails->error) || !empty($chargeDetails->errors) ||
            empty($chargeDetails) || empty($chargeDetails->charges)) {

            $rechargeError  = "Unknown error";
            if (!empty($chargeDetails->error)) {

                $rechargeError = json_encode($chargeDetails->error);
            } else if (!empty($chargeDetails->errors)) {

                $rechargeError = json_encode($chargeDetails->errors);
            } else if (empty($chargeDetails) || empty($chargeDetails->charges)) {

                $rechargeError = 'Failed to get the charge details from recharge';
            }
            $returnData = [
                'error' => $rechargeError
            ];

            Log::channel('giftRecipientPasswordResetLog')->info("getGiftRecipientPasswordResetStatus: Failed to get the 
            charge details from the recharge(subscriptionId:".$purchaserSubscriptionId."), Error: (".$rechargeError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $shopifyRecipientCustomerEmailFromPurchaserCharge = '';
        $purchaserProperties = isset($chargeDetails->charges[0]->line_items[0]->properties) ? 
        $chargeDetails->charges[0]->line_items[0]->properties : '';

        if (isset($purchaserProperties)) {

            foreach ($purchaserProperties as $propKey => $propValue) {

                if(trim($propValue->name) == 'giftee_email') {

                    $shopifyRecipientCustomerEmailFromPurchaserCharge = trim($propValue->value);
                }
            }
        }
        if ($emailAddress != $shopifyRecipientCustomerEmailFromPurchaserCharge) {

            $returnData = [
                'error' => 'Failed to match the gift recipient with the recipient in the purchaser order',
            ];
            Log::channel('giftRecipientPasswordResetLog')->info("getGiftRecipientPasswordResetStatus: Failed to match the 
            gift recipient with the recipient in the purchaser order, subscriptionId: ".$purchaserSubscriptionId);
            return $this->returnData(600, "error", $returnData);
        }

        // Get Shopify customer details by Id
        $shopifyCustomerDetails = $this->customerObj->getCustomerDetails($recipientShopifyCustomerId);
        if (!empty($shopifyCustomerDetails['error']) || !empty($shopifyCustomerDetails['errors']) ||
            empty($shopifyCustomerDetails) || empty($shopifyCustomerDetails['customer'])) {

            $shopifyError  = "Unknown error";
            if (!empty($shopifyCustomerDetails['error'])) {

                $shopifyError = json_encode($shopifyCustomerDetails['error']);
            } else if (!empty($shopifyCustomerDetails['errors'])) {

                $shopifyError = json_encode($shopifyCustomerDetails['errors']);
            } else if (empty($shopifyCustomerDetails) || empty($shopifyCustomerDetails['customer'])) {

                $shopifyError = 'Failed to get the customer details from Shopify';
            }
            $returnData = [
                'error' => $shopifyError
            ];

            Log::channel('giftRecipientPasswordResetLog')->info("getGiftRecipientPasswordResetStatus: Failed to get 
            the customer details from Shopify(CustomerId:".$recipientShopifyCustomerId."), Error: (".$shopifyError.")");
            return $this->returnData(600, "error", $returnData);
        }

        $shopifyCustomerExistingTags = '';
        $customerStatus = !empty($shopifyCustomerDetails['customer']['state']) ? 
        $shopifyCustomerDetails['customer']['state'] : '';
        $shopifyCustomerExistingTags = !empty($shopifyCustomerDetails['customer']['tags']) ? 
        trim($shopifyCustomerDetails['customer']['tags']) : '';

        if (trim($customerStatus) != 'enabled') {

            $returnData = [
                'error' => 'The gift recipient Shopify account is not enabled',
            ];
            Log::channel('giftRecipientPasswordResetLog')->info("getGiftRecipientPasswordResetStatus: The gift recipient 
            Shopify account is not enabled, endpoint request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        if (empty($shopifyCustomerExistingTags)) {

            $returnData = [
                'message' => 'Successfully get the password reset status',
                'data' => [
                    'passwordSet' => 'false'
                ]
            ];
    
            Log::channel('giftRecipientPasswordResetLog')->info("getGiftRecipientPasswordResetStatus: ".
            json_encode($returnData));
            return $this->returnData(200, "success", $returnData);
        }

        $passwordSetTagStatus = 0;
        $explodeTags = explode(",", $shopifyCustomerExistingTags);
        $explodeTags = array_map('trim', $explodeTags);
        $explodeTags = array_map('strtolower', $explodeTags);
        if (in_array("passwordset", $explodeTags)) {
            $passwordSetTagStatus = 1;
        }

        $returnData = [
            'message' => 'Successfully get the password reset status',
            'data' => [
                'passwordSet' => ($passwordSetTagStatus == 1) ? 'true' : 'false'
            ]
        ];

        Log::channel('giftRecipientPasswordResetLog')->info("getGiftRecipientPasswordResetStatus: ".
        json_encode($returnData));
        return $this->returnData(200, "success", $returnData);
    }

    /**
     * Response function
     */
    public function returnData($status, $message, $error, $jsonCallback = null) {

        $returnData = array(
            'status' => $status,
            'message' => $message,
            'body' => $error

        );
        if (!empty($jsonCallback)) {

            echo $jsonCallback . '(' . json_encode($returnData) . ')';
            die();
        } else {

            return response()->json($returnData);
        }
    }

}
