<?php

namespace App\Http\Controllers;

use Log;
use Carbon\Carbon;
use App\Model\dataPostInternal;
use App\Helper\Helper;
use Illuminate\Http\Request;

class quizCartOffLoadController extends Controller {
    
    private $dataPostObj;

    public function __construct() {

        $this->dataPostObj = new dataPostInternal();
    }

    /**
     * Quiz cart offLoad data post to internal
     */
    public function quizCartOffload(Request $request) {

        $requestData = $request->input('data');

        if (empty($requestData)) {

            $returnData = [
                'error' => 'Invalid endpoint request',
            ];            
            return $this->returnData(600, "error", $returnData);
        }
        $receivedData = json_decode($requestData);

        $quizId = !empty($receivedData->QuizId) ? trim($receivedData->QuizId) : '';
        $cartProducts = isset($receivedData->Cart) ? $receivedData->Cart : '';

        if (empty($quizId)) {

            $returnData = [
                'error' => 'Required parameter quizID is missing in the endpoint request',
            ];
            Log::channel('quizCartOffLoadLogging')->info("Required parameter quizID is missing 
            in the endpoint request, request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        if (empty($cartProducts)) {

            $returnData = [
                'error' => 'Required product details are missing in the endpoint request',
            ];
            Log::channel('quizCartOffLoadLogging')->info("Required product details are missing in 
            the endpoint request, request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        $cartProductDataStatus = 1;

        foreach ($receivedData->Cart as $productKey => $productValue) {

            if (empty($productValue->BulkItem) || empty($productValue->Qty) || empty($productValue->VariantID)) {

                $cartProductDataStatus = 0;
                break;
            }
        }

        if ($cartProductDataStatus == 0) {

            $returnData = [
                'error' => 'Required parameters are missing from the cart products in the request',
            ];
            Log::channel('quizCartOffLoadLogging')->info("The cart products in the endpoint request 
            are empty, request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        $internalPostResponse = $this->dataPostObj->postQuizCartOffLoadData($requestData);
        $internalPostResponse = !empty($internalPostResponse) ? trim(strtolower($internalPostResponse)) : '';

        if ($internalPostResponse != 'success') {

            $returnData = [
                'error' => 'Failed to post the request to the internal URL',
            ];
            Log::channel('quizCartOffLoadLogging')->info("Failed to post the request to the internal URL, 
            request: ".json_encode($receivedData));
            return $this->returnData(600, "error", $returnData);
        }

        $returnData = [
            'message' => 'Successfully posted the given request to the internal URL'
        ];

        Log::channel('quizCartOffLoadLogging')->info("Successfully posted the given request in the URL
        (endpoint request:".json_encode($receivedData).")");
        return $this->returnData(200, "success", $returnData);
    }

    /**
     * Response function
     */
    public function returnData($status, $message, $error, $jsonCallback = null) {

        $returnData = array(
            'status' => $status,
            'message' => $message,
            'body' => $error

        );
        if (!empty($jsonCallback)) {

            echo $jsonCallback . '(' . json_encode($returnData) . ')';
            die();
        } else {

            return response()->json($returnData);
        }
    }

}
