<?php
namespace App\Model;
use Illuminate\Database\Eloquent\Model;

class RechargeApi extends Model {

    private $rechargeUrl;
    private $accessToken;
    private $rechargeObj;
    private $rechargeApiKey;

    function __construct($type = NULL) {

        $this->rechargeUrl = env('RECHARGE_URL', true);
        if ($type == 'gift') {
            $this->rechargeApiKey = env('RECHARGE_API_KEY',true);
        } else if ($type == 'giftAndGetAllSubscriptions') {
            $this->rechargeApiKey = env('RECHARGE_API_KEY_V2',true);
        } else {
            $this->rechargeApiKey = env('RECHARGE_API_KEY',true);
        }
    }

    public function getChargeById($id){

        $method = 'GET';
        $path = $this->rechargeUrl . 'charges/'.$id;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * Get charges by start and end date
    */
    public function getChargesByMaxMinDates($min, $max, $page, $limit) {
        $method = 'GET';
        $path = $this->rechargeUrl . '/charges?date_min='.$min.'&date_max='.$max.'&status=QUEUED&page='.$page.'&limit='.$limit;
        
        // echo $path;die;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * update subscription by ID
     */
    function updateSubscription($subscription_id, $data = NULL) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/subscriptions/'.$subscription_id;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get subscription by subscription_id
     */
    function getSubscription($subscription_id) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/subscriptions/' . $subscription_id;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get subscription by subscription_id
     */
    function activateSubscription($subscription_id) {

        $method = 'POST';
        $path = $this->rechargeUrl . '/subscriptions/' . $subscription_id . '/activate';
        $data="{}";
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * create recharge checkout
     */
    function createCheckout($data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/checkouts';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * update recharge checkout
     */
    function updateCheckout($token, $data) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/checkouts/'.$token;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get recharge checkout
     */
    function retrieveCheckout($token, $status = 'checkout') {

        $method = 'GET';
        if($status == 'checkout') {
            $path = env('RECHARGE_URL', true).'/checkouts/'.
            $token;
        } else {
            $path = env('RECHARGE_URL', true).'/checkouts/'.
                $token.'/shipping_rates';
        }
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * update shipping line item
     */
    function updateShippingLine($token, $data) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/checkouts/'.$token;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get customer details by customer id
     */
    function getRechargeCustomerDetailsByID($customerID) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/customers/'.$customerID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get customer details by customer email
     */
	function getRechargeCustomerDetailsByEmail($email) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/customers?email='.$email;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get subscription by customer id
     */
    function getSubscriptionByCustomerID($customerID) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/subscriptions?customer_id='.$customerID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get customer address by address id
     */
    function getAddressByAddressID($addressId) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/addresses/'.$addressId;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get shipping rate by address
     */
    function updateShippingRateByAddress($addressId, $data) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/addresses/'.$addressId;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get all subscription by customer id
     */
    function getAllSubscriptionsByCustomerID($customerID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/subscriptions?customer_id='.$customerID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * update subscription using upsell addon
     */
    function updateUpsellSubscription($data = NULL) {

        $method = 'GET';
        $path = env('UPDATE_SUBSCRIPTION_URL', true).'updateSubscriptionLineItemProperties';
        $response = $this->callEndpoint($method, $path, $data);
	$response = trim($response,")");
        $response = ltrim($response,"update(");
        return json_decode($response,true);
    }

    /**
     * get all active subscriptions by customer id
     */
    function getAllActiveSubscriptionsByCustomerID($customerID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/subscriptions?customer_id='.$customerID.'&status=active';
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * change subscription next charge date
     */
    function updateSubscriptionChargeDate($subscriptionID, $data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/subscriptions/'.$subscriptionID.'/set_next_charge_date';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get charge details by addressID
     */
    function getOtpChargeDetails($addressID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/charges?address_id='.$addressID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * create subscription in recharge
     */
    function createSubscription($data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/subscriptions';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * get charge details by subscriptionID
     */
    function getChargeDetailsBySubscriptionID($subscriptionID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/charges?subscription_id='.$subscriptionID;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * get discount details by discount code
     */
    function getDiscountDetailsByDiscountCode($discountCode) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/discounts?discount_code='.$discountCode;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * apply discount to a charge by discount code
     */
    function applyDiscountToChargeByDiscountCode($chargeID, $data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/charges/'.$chargeID.'/apply_discount';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * Get all the subscriptions by the min date
     */
    function getAllSubscriptionsByMinDate($minDate, $pageNo, $limit) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/subscriptions?created_at_min='.$minDate.'&page='.$pageNo.'&limit='.$limit;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * Get charge details by shopify order ID
     */
    function getChargeDetailsByShopifyOrderId($shopifyOrderId) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/charges?shopify_order_id='.$shopifyOrderId;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * Refund recharge charge by ID
     */
    function refundRechargeChargeById($chargeId, $data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/charges/'.$chargeId.'/refund';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * Get orders by recharge address ID
     */
    function getRechargeOrdersByAddressId($addressId) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/orders?address_id='.$addressId;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * Get customer by shopify customer ID
     */
    function getCustomerDetailsByShopifyCustomerId($shCustomerId) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/customers?shopify_customer_id='.$shCustomerId;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * Cancel subscription
     */
    function cancelSubscription($subscriptionId, $data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/subscriptions/'.$subscriptionId.'/cancel';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * Update subscription address
     */
    function updateSubscriptionAddress($addressId, $data) {

        $method = 'PUT';
        $path = env('RECHARGE_URL', true).'/addresses/'.$addressId;
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * Update order scheduled date
     */
    function updateOrderScheduledDate($orderId, $data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/orders/'.$orderId.'/change_date';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * Get customer details by customer hash
     */
    function getRechargeCustomerDetailsByHash($hash) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/customers?hash='.$hash;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * Get orders by customer Id
     */
    public function getOrdersByCustomerId($customerId, $limit = NULL, $cursorNextLink = NULL) {

        $method = 'GET';
        if(!empty($cursorNextLink)) {
            $path = $cursorNextLink;
        } else {
            $path = $this->rechargeUrl . '/orders?customer_id='.$customerId.'&status=success&limit='.$limit;
        }
        $response = $this->callPaging($method, $path);
        return $response;
    }

    /**
     * Create a customer in recharge
     */
    function createRechargeCustomer($data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/customers';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * Get the recharge order by Shopify order Id
     */
    function getRechargeOrderByShopifyOrderId($orderId) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/orders?shopify_order_id='.$orderId;
        $response = $this->call($method, $path);
        return json_decode($response);
    }

    /**
     * Cancel recharge subscription by ID
     */
    function cancelRechargeSubscription($subscriptionId, $data) {

        $method = 'POST';
        $path = env('RECHARGE_URL', true).'/subscriptions/'.$subscriptionId.'/cancel';
        $response = $this->call($method, $path, $data);
        return json_decode($response);
    }

    /**
     * Get queued charges based on Min and Max date
     */
    public function getQueuedChargesByMaxMinDates($limit, $url, $minDate, $maxDate) {

        $method = 'GET';

        if(!empty($url)) {
            $path = $url;
        } else {
            $path = $this->rechargeUrl . '/charges?status=queued&date_min='.$minDate.'&date_max='.$maxDate.'&limit='.$limit;
        }
        $response = $this->callPaging($method, $path);
        return $response;
    }

    /**
     * get all subscription by customer id
     */
    function getAllSubscriptionsByCustomerIDV2($customerID) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/subscriptions?customer_id='.$customerID;
        $response = $this->callV2($method, $path);
        return $response;
    }

    /**
     * get subscription by subscription_id
     */
    function getSubscriptionV2($subscription_id) {

        $method = 'GET';
        $path = $this->rechargeUrl . '/subscriptions/' . $subscription_id;
        $response = $this->callV2($method, $path);
        return $response;
    }

    /**
     * Get orders by recharge address ID
     */
    function getRechargeOrdersByAddressIdV2($addressId) {

        $method = 'GET';
        $path = env('RECHARGE_URL', true).'/orders?address_id='.$addressId;
        $response = $this->callV2($method, $path);
        return $response;
    }

    /**
     * CURL call
     */
    function call($method, $path, $data = []) {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        if ($method !== 'GET') {

            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }

        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "cache-control: no-cache",
            "x-recharge-access-token: " . $this->rechargeApiKey,
            "content-type: application/json",
        ));

        $response = curl_exec($curl);
        return $response;

    }

    /**
     * CURL call V2
     */
    public function callV2($method, $path, $data = []) {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($curl, CURLOPT_HEADER, 1);

        if ($method !== 'GET') {

            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }

        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "cache-control: no-cache",
            "x-recharge-access-token: " . $this->rechargeApiKey,
            "content-type: application/json",
        ));
        
        $response = curl_exec($curl);
        $headers = $this->get_headers_from_curl_response($response);
        // $pageLink = !empty($headers['Link']) ? $headers['Link'] : "";
        // $next = $this->getNextLink($pageLink);
        $rateLimitReached = ["33/40", "34/40", "35/40", "36/40", "37/40", "38/40"];
        if (isset($headers['X-Recharge-Limit']) && !empty($headers['X-Recharge-Limit'])) {
            if (in_array(trim($headers['X-Recharge-Limit']),  $rateLimitReached)) {
                sleep(5);
            }
        }
        $httpStatusCode = !empty($headers['http_code']) ? $headers['http_code'] : "";
        $body = str_replace(substr($response, 0, strpos($response, "\r\n\r\n"))."\r\n\r\n", "", $response);
        $body = json_decode($body);
        //$body["next"] = $next;
        if (empty($body) || (isset($body->error) && !empty($body->error)) || 
        (isset($body->errors) && !empty($body->errors))) {

            $body['http_status_code'] = $httpStatusCode;
            $body['request'] = $path;
        }
        return $body;
    }

    /**
     * CURL call for update the upsell subscription
     */
    function callEndpoint($method, $path, $data = []) {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path.'?callback=update&data='.urlencode($data));
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        //curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        //curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        //curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);

        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "content-type: application/json",

        ));

        $response = curl_exec($curl);
        return $response;

    }

    /**
    * CURL call to create checkout
    */
    function callEndpointToCreateCheckout($method, $path, $data = []) {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path.'?callback=create&data='.urlencode($data));
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        //curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        //curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        //curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        
            curl_setopt($curl, CURLOPT_HTTPHEADER, array(
        "content-type: application/json",
        ));
        
        $response = curl_exec($curl);
        return $response;
    }

    /**
     * Cursor pagination
     */
    public function callPaging($method, $path, $data = []) {

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $path);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_ENCODING, '');
        curl_setopt($curl, CURLOPT_MAXREDIRS, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 30);
        curl_setopt($curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        curl_setopt($curl, CURLOPT_CUSTOMREQUEST, $method);
        curl_setopt($curl, CURLOPT_HEADER, 1);

        if ($method !== 'GET') {

            curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        }

        curl_setopt($curl, CURLOPT_HTTPHEADER, array(
            "cache-control: no-cache",
            "x-recharge-access-token: " . $this->rechargeApiKey,
            "content-type: application/json",
        ));
        
        $response = curl_exec($curl);
        $headers = $this->get_headers_from_curl_response($response);
        $pageLink = !empty($headers['Link']) ? $headers['Link'] : "";
        $next = $this->getNextLink($pageLink);
        $rateLimitReached = ["30/40", "31/40", "32/40", "33/40", "34/40", "35/40"];
        if (isset($headers['X-Recharge-Limit']) && !empty($headers['X-Recharge-Limit'])) {
            if (in_array(trim($headers['X-Recharge-Limit']),  $rateLimitReached)) {
                sleep(10);
            }
        }
        $httpStatusCode = !empty($headers['http_code']) ? $headers['http_code'] : "";
        $body = str_replace(substr($response, 0, strpos($response, "\r\n\r\n"))."\r\n\r\n", "", $response);
        $body = json_decode($body, true);
        $body["next"] = $next;
        if (empty($body) || (isset($body['error']) && !empty($body['error'])) || 
        (isset($body['errors']) && !empty($body['errors']))) {

            $body['http_status_code'] = $httpStatusCode;
            $body['request'] = $path;
        }
        return $body;
    }

    /**
     * Get header details from the API response
     */
    public function get_headers_from_curl_response($response) {

        $headers = array();

        $header_text = substr($response, 0, strpos($response, "\r\n\r\n"));
        foreach (explode("\r\n", $header_text) as $i => $line) {
            if ($i === 0) {

                $headers['http_code'] = $line;
            } else {

                list ($key, $value) = explode(': ', $line);

                $headers[$key] = $value;
            }
        }
        return $headers;
    }

    /**
     * Get next cursor page link from the API response header
     */
    public function getNextLink($pageLink) {

        $nextUrl = "";
        if (!empty($pageLink)) {

            if ((strpos($pageLink, 'rel="previous"') !== false) && (strpos($pageLink, 'rel="next"') !== false)) {

                $all_urls = explode(", ", $pageLink);
                $nextUrl = $all_urls[0];
            }
            elseif (strpos($pageLink, 'rel="next"') !== false){

                $nextUrl = $pageLink;
            } else {

                $nextUrl = '';
            }

            $nextUrl = explode(";", $nextUrl);
            $nextUrl = $nextUrl[0];
            $nextUrl = rtrim($nextUrl,">");
            $nextUrl = ltrim($nextUrl,"<");
        }
        return $nextUrl;
    }
}
